/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Length.cpp,v 1.5 2004/02/19 12:27:36 yoshi Exp $ */ 
#include "Length.h"

Length::Length() : m_value(-1), m_center(-1), m_innerStart(0), m_innerEnd(0) {;}
Length::Length(const Length &o)
	: m_value(o.m_value), m_center(o.m_center),
	  m_innerStart(o.m_innerStart), m_innerEnd(o.m_innerEnd)	{;}

Length::Length(Type v) : m_value(v), m_center(v/2), m_innerStart(0), m_innerEnd(v) {;}
Length::Length(Type v, Type c) : m_value(v), m_center(c), m_innerStart(0), m_innerEnd(v) {;}


void Length::value(Type v)
{
	m_value = v;
	m_center = v/2;
	m_innerStart = 0;
	m_innerEnd = v;
}


Length &Length::operator=(const Length &o)
{
	this->m_value = o.m_value;
	this->m_center = o.m_center;
	this->m_innerStart = o.m_innerStart;
	this->m_innerEnd = o.m_innerEnd;
	return *this;
}

#define MAX(a, b) ( ((a) > (b))? (a): (b) )

#define ADD(l1, l2, func) MAX((l1).func(), (l2).func())

#define ADD_SC(l1, l2) ADD(l1, l2, start_center)
#define ADD_EC(l1, l2) ADD(l1, l2, end_center)

Length &Length::operator+=(const Length &o)
{
	Length::Type sc = ADD_SC(*this, o);
	Length::Type ec = ADD_EC(*this, o);
	this->value(sc + ec);
	this->center(sc);
	return *this;
}

Length &Length::operator+=(int n)
{
	m_value += n;
	m_center += n/2;
	m_innerStart = 0;
	m_innerEnd = m_value;
	return *this;
}

#if 0
Length operator+(const Length &l1, const Length &l2)
{
	const Length::Type sc = ADD_SC(l1, l2);
	const Length::Type ec = ADD_EC(l1, l2);

	return Length(sc + ec, sc);
}
#endif

#ifdef LENGTH_TEST1

#include <stdio.h>

int main()
{
	Length l1(4, 1);
	Length l2(6, 5);

	Length l = l1 + l2;
	printf("value : %d\n", l.value());
	printf("center : %d\n",l.start_center());

	l1 += l2;
	printf("value : %d\n", l1.value());
	printf("center : %d\n",l1.start_center());

	return 0;
}
#endif
	
#ifdef LENGTH_TEST2

#include <stdio.h>

int main()
{
	Length l1(4, 1);
	Length l2;
	l2 = l1;


	printf("value : %d\n", l1.value());
	printf("center : %d\n",l1.start_center());

	printf("value : %d\n", l2.value());
	printf("center : %d\n",l2.start_center());

	return 0;
}
#endif
	



