/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Math2DLoader.cpp,v 1.18 2004/03/23 05:07:33 yoshi Exp $ */
#include <string.h>
#include "Math2DLoader.h"
#include "TextFileLineReadIterator.h"
#include "TextStdinLineReadIterator.h"
#include "StringBlockBuilder.h"
#include "StringBlockWalker.h"
#include "ScriptBlockIterator.h"
#include "Math2DParser.h"
#include "ExceptionDef.h"
#include "ParserException.h"
#include "Equation.h"

#ifdef READ_HTTP_TEXT
#include "UrlTextLineReadIterator.h"
#endif

Math2DLoader::Math2DLoader() : m_err(0), m_line(-1) {;}

Math2DLoader::~Math2DLoader() {	free_(); }

void	Math2DLoader::free_()
{
	EquationContainerIterator itr;
	itr.set(&m_equations);
	while (itr.hasMoreElements()) {
		Equation *eq = itr.element();
		delete eq;
	}
	m_equations.clear();
}


#define STRING_EQUALS(str, key) (strncmp(str, key, strlen(key)) == 0)
#define HTTP_STR	"http"

static char *convertCharCode(const char *fname)
{
	static char buf[1024];
	char tmp[1024];
	strcpy(tmp, fname);

	char *pbuf = buf;
	char *p = tmp;
	char *to = pbuf;
	while (*p) {
		if (strncmp(p, "%20", 3) == 0) {
			*to = ' '; to++;
			p += 3;
		} else {
			*to = *p;
			to++; p++;
		}
	}
	*to = 0;
	return pbuf;
}


static TextLineReadIterator *createFileReader(const char *fname)
{
	TextLineReadIterator *p;
	if (0);
	else if (!fname) { p = new TextStdinLineReadIterator; }
#ifdef READ_HTTP_TEXT
	else if (STRING_EQUALS(fname, HTTP_STR)) { p = new UrlTextLineReadIterator(fname); }
#endif
	else {
		// 040310
		// uEUnꂽ[Jt@ĆA
		// 󔒂%20ɂȂĂ邱ƂB
		char *to = convertCharCode(fname);
		p = new TextFileLineReadIterator(to);
	}
	return p;
}

bool	Math2DLoader::load(const char *fname)
{
	TextLineReadIterator *itr = createFileReader(fname);
	bool ret = load(*itr);
	delete itr;
	return ret;
}

bool	Math2DLoader::load(TextLineReadIterator &itr)
{
	bool	ret = true;

	DTRY {
		StringBlockBuilder		build;
		build.set(&itr);
		StringBlockBuilder::Container	block_container;
		build(block_container);
		m_line = itr.line();

		StringBlockBuilder::ContainerIterator citr;
		citr.set(&block_container);

		while (citr.hasMoreElements()) {
			StringBlock *block = citr.element();
			StringBlockWalker	walker; walker.set(block);
			Math2DParser parse; parse.set(&walker);
			Equation * eq = parse();
			if (eq) { m_equations.pushBack(eq); }
		}
	
	} DCATCH_START(ParserIllegalDescriptionException) {
		if (m_err) {
			m_err->write(e.msg(), e.line());
		}
	} DCATCH_START(Exception) {
		if (m_err) {
			m_err->write(e.msg());
		}
	} DCATCH_END();
	return ret;
}

