/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
#include "Math2DWinDrawer.h"
#include "ViewBuffer.h"
#include "DrawContext.h"
#include "Math2DContextCreatorBase.h"
#include "HilightSwitch.h"

#include "eqcheck.h"

static void initErrBuf();
static void outputErr(HDC hDC);

Math2DWinDrawer::Math2DWinDrawer(HDC hDC, int width, int height, Math2DContextCreatorBase &creator)
	: EquationRegister(), m_updateListener(0), m_errWriter(0), m_loader(0), m_buffer(0), m_origBuffer(0),
	  m_context(0), m_contextCreator(creator)
{
	init(hDC, width, height);
}

void	Math2DWinDrawer::init(HDC hDC, int width, int height)
{
	if (m_origBuffer) { delete m_origBuffer; m_origBuffer = 0; }
	m_origBuffer = new ViewBuffer(hDC, width, height);
}

#define FREE(p) if (p) { delete p; p = 0; } 

void	Math2DWinDrawer::free_()
{
	FREE(m_loader);
	FREE(m_origBuffer);
	FREE(m_context);
}

void	Math2DWinDrawer::setErrorWriter(Math2DLoader *loader)
{
	if (m_errWriter) { m_loader->set(m_errWriter); }
	else { m_loader->set(this);}
}

DrawContext * Math2DWinDrawer::drawContext()
{
	if (!m_context) { m_context = m_contextCreator.create(this, &m_dictionary); }
	return m_context;
}

#define MAX(a, b) (((a) > (b))? (a): (b))

const int START_X = 0;
const int START_Y = 0;
const int HEIGHT_MARGIN = 5; 
const int WIDTH_MARGIN = 10;



Equation::Size&	Math2DWinDrawer::calcSize(Math2DLoader::EquationContainer &eqs)
{
	static Equation::Size sz;
	assert(m_updateListener);
	assert(m_origBuffer);

	DrawContext *context = drawContext();
	// DCgp̂ŁA`ȂĂݒ肵ĂB
	context->buffer = m_origBuffer; 
	
	Length	center;
	{
		Math2DLoader::EquationContainerIterator itr;
		itr.set(&eqs);
		while (itr.hasMoreElements()) {
			Equation *eq = itr.element();
			context->point.x.setStart(START_X);
			context->point.y.setStart(0); // Ƃ肠ݒ肷KvB
			const Length &l = eq->centerWidth(context);
			center += l;
		}
	}
	context->centerWidth(center);

	Length	width;
	Length	height;

	int cnt = 0;
	Math2DLoader::EquationContainerIterator itr; itr.set(&eqs);
	while (itr.hasMoreElements()) {
		if (cnt > 0) { height += HEIGHT_MARGIN; }
		Equation *eq = itr.element();
		context->point.x.setStart(START_X);
		context->point.y.setStart(0); // Ƃ肠ݒ肷KvB
		Equation::Size *sz = eq->size(context);
		width += sz->widthLength();
		height += sz->height();
		cnt++;
	}

	width += WIDTH_MARGIN;
	height += HEIGHT_MARGIN; // window ƁAY\ȂB
	sz.set(width.value(), height.value());
	sz.setCenter(width.start_center(), height.start_center());

	m_updateListener->resize(sz.widthLength(), sz.heightLength());
	return sz;
}

#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

bool	Math2DWinDrawer::draw(const char *filename)
{
	if (!m_origBuffer) { return false; }

	m_loader = new Math2DLoader;
	setErrorWriter(m_loader);
	if (!m_loader->load(filename)) { return false; }

	Math2DLoader::EquationContainer &eqs = m_loader->getContainer();
	Equation::Size &whole = calcSize(eqs);
	Math2DLoader::EquationContainerIterator itr; itr.set(&eqs);

	DrawContext *context = drawContext();
	int     y = START_Y;
	int	cnt = 0;
	while (itr.hasMoreElements()) {
		if (cnt > 0) { y += HEIGHT_MARGIN; }
		context->start(m_origBuffer);

		context->point.x.setCenter(whole.widthLength().start_center()); // Sɕ\
		context->point.y.setStart(y); // Ƃ肠ݒ肷KvB

		Equation *eq = itr.element();

		DEBUG_DRAW_RECT(eq, context);
		Equation::Size *sz = eq->size(context);
		context->point.x.setCenter(context->point.x.center(sz->widthLength()));
		context->point.y.setCenter(context->point.y.center(sz->heightLength()));
		eq->draw(context);

		y += sz->height();

		context->end();
		cnt++;
	}
	m_buffer = m_origBuffer;
	outputErr(m_buffer->getDC());
	return true;
}

ViewBuffer * Math2DWinDrawer::checkHilight(char *tags)
{
	if (!tags || strlen(tags) <= 0) { return m_origBuffer; }
	ViewBuffer *buffer = m_bufferMgr.find(tags);
	if (!buffer) { 

		buffer = new ViewBuffer(*m_origBuffer);
		m_bufferMgr.set(tags, buffer);

		HilightSwitch *hs = drawContext()->hilightSwitch;
		hs->on(buffer->getDC());
		for (char *p=tags; *p; p++){ 
			m_taggedMap.draw(*p, buffer);
		}
		hs->off(buffer->getDC());
	}
	return buffer;
}


void Math2DWinDrawer::hilight(char tag)
{
	static char tags[2];
	tags[0] = tag; tags[1] = 0;
	hilight(tags);
}

void Math2DWinDrawer::hilight(const char *tags)
{
	static char buf[256];
	strcpy(buf, tags);
	ViewBuffer *newbuffer = checkHilight(buf);
	if (newbuffer != m_buffer) { redraw(newbuffer);	}
}


char* Math2DWinDrawer::inLinkRect(int x, int y)
{
	ViewBuffer *newbuffer;
	char *tags = m_linkMgr.link(x, y);
	if (tags) {
		newbuffer = checkHilight(tags);
	} else {
		newbuffer = m_origBuffer;
	}
	if (newbuffer != m_buffer) { redraw(newbuffer);	}
	return tags;
}

static char err_buf[2048] = "";
static void initErrBuf()
{
	err_buf[0] = 0;
}
static void outputErr(HDC hDC)
{
	if (*err_buf) {
		::TextOut(hDC, 10, 500, err_buf, strlen(err_buf));
	}
}

void Math2DWinDrawer::write(const char *msg, int line)
{
	sprintf(err_buf, "%d : %s", line, msg);
}

void Math2DWinDrawer::clear()
{
	if (m_origBuffer) { m_origBuffer->clearBuffer(); }
	m_taggedMap.clear();
	m_linkMgr.clear();
	m_bufferMgr.clear();
	FREE(m_loader);
	FREE(m_context);
	m_buffer = 0;
	initErrBuf();
	m_dictionary.clear();
}

#include "BracketEquation.h"
#include "EqualEquation.h"
#include "GreaterEquation.h"
#include "GreekCharacter.h"
#include "IntegralEquation.h"
#include "LessEquation.h"
#include "MatrixEquation.h"
#include "MinusEquation.h"
#include "NoOperatorConnection.h"
#include "OperatorConnection.h"
#include "ParamEquation.h"
#include "PlusEquation.h"
#include "PowerEquation.h"
#include "SubScriptEquation.h"
#include "SummationEquation.h"
#include "TaggedEquation.h"
#include "VectorEquation.h"
#include "LinkEquation.h"
#include "HatEquation.h"
#include "TildeEquation.h"
#include "FractionEquation.h"
#include "ProductEquation.h"
#include "IntegralSymbolEquation.h"
#include "ProductSymbolEquation.h"
#include "SummationSymbolEquation.h"
#include "ConditionEquation.h"
#include "AlignmentEquation.h"
#include "DotEquation.h"
#include "AsteriskEquation.h"
#include "PrimeEquation.h"
#include "SpaceEquation.h"
#include "NotImplementedEquation.h"
#include "DefinitionEquation.h"
#include "ReferenceEquation.h"
#include "EquationEnumeratorEquation.h"
#include "CasesEquation.h"
#include "ConditionValueConnection.h"
#include "RawDescriptionEquation.h"
#include "NoNumberEquation.h"
#include "VectorParamEquation.h"

#include "BackSlashEquationImpl.h"
#include "ExtendedRangeEquationImpl.h"
#include "ComparativeOperatorEquationImpl.h"
#include "BinominalOperatorEquationImpl.h"
#include "ContentEquationImpl.h"
#include "ArrowEquationImpl.h"

#include "BracketEquationDCDrawer.h"
#include "GreekCharacterDCDrawer.h"
#include "InfinityEquationDCDrawer.h"
#include "IntegralEquationDCDrawer.h"
#include "LogEquationDCDrawer.h"
#include "MatrixEquationDCDrawer.h"
#include "NoOperatorConnectionDCDrawer.h"
#include "OperatorConnectionDCDrawer.h"
#include "ParamEquationDCDrawer.h"
#include "PowerEquationDCDrawer.h"
#include "SubScriptEquationDCDrawer.h"
#include "SummationEquationDCDrawer.h"
#include "TaggedEquationDCDrawer.h"
#include "VectorEquationDCDrawer.h"
#include "LinkEquationDCDrawer.h"
#include "SquareRootEquationDCDrawer.h"
#include "FractionEquationDCDrawer.h"
#include "ProductEquationDCDrawer.h"
#include "ProductSymbolEquationDCDrawer.h"
#include "IntegralSymbolEquationDCDrawer.h"
#include "SummationSymbolEquationDCDrawer.h"
#include "CharSymbolEquationDCDrawerImpl.h"
#include "CharSymbolSuperSubScriptEquationDCDrawer.h"
#include "ConditionEquationDCDrawer.h"
#include "AlignmentEquationDCDrawer.h"
#include "TrigonometricFunctionDCDrawer.h"
#include "DoubleUpArrowEquationDCDrawer.h"
#include "DoubleDownArrowEquationDCDrawer.h"
#include "DeterminantEquationDCDrawer.h"
#include "DotEquationDCDrawer.h"
#include "SpaceEquationDCDrawer.h"
#include "NotImplementedEquationDCDrawer.h"
#include "VectorParamEquationDCDrawer.h"
#include "BarEquationDCDrawer.h"
#include "DefinitionEquationDCDrawer.h"
#include "ReferenceEquationDCDrawer.h"
#include "EquationEnumeratorEquationDCDrawer.h"
#include "CasesEquationDCDrawer.h"
#include "ConditionValueConnectionDCDrawer.h"
#include "RawDescriptionEquationDCDrawer.h"
#include "ExpEquationDCDrawer.h"
#include "PartialEquationDCDrawer.h"
#include "NotSupportedEquationDCDrawer.h"
#include "NotSupportedContentEquationDCDrawer.h"
#include "EquationDummyDCDrawer.h"
#include "MiscEquationDCDrawerImpl.h"
#include "EquationImageObjectDCDrawerImpl.h"
#include "EquationCharacterDCDrawerImpl.h"
#include "OverlineEquationDCDrawer.h"
#include "UnderlineEquationDCDrawer.h"
#include "OverwriteCharacterEquationDCDrawerImpl.h"
#include "OverwriteImageEquationDCDrawerImpl.h"

#include "FontManager.h"

void Math2DWinDrawer::setEquation(TaggedEquation *eq, DrawContext *context)
{
	char *tag = eq->tag();
	m_taggedMap.add(tag[0], eq, context);
}

void Math2DWinDrawer::setEquation(LinkEquation *link, DrawContext *context)
{
	Equation::Size *sz = ((Equation*)link)->size(context);
	int sx = context->point.x.start(sz->width());
	int sy = context->point.y.start(sz->height());
	int ex = context->point.x.end(sz->width());
	int ey = context->point.y.end(sz->width());
	m_linkMgr.set(link, sx, sy, ex, ey);

	char *name = link->name();
	m_taggedMap.add(name[0], link, context);
}

const int DRAWER_SIZE = 512;
static int		s_drawerCnt = 0;
static EquationDrawer *	s_drawers[DRAWER_SIZE];

#define SET_EQUATION_DRAWER(EQ) \
	assert(s_drawerCnt < DRAWER_SIZE); EQ::setDrawer((s_drawers[s_drawerCnt++] = new EQ##DCDrawer));
#define SET_EQUATION_DRAWER2(EQ, DRAWER) \
	assert(s_drawerCnt < DRAWER_SIZE); \
	EQ::setDrawer((s_drawers[s_drawerCnt++] = new DRAWER));

void Math2DWinDrawer::initDrawer()
{
	SET_EQUATION_DRAWER(BracketEquation);
	SET_EQUATION_DRAWER(EqualEquation);
	SET_EQUATION_DRAWER(GreaterEquation);
	SET_EQUATION_DRAWER(GreekCharacter);
	SET_EQUATION_DRAWER(IntegralEquation);
	SET_EQUATION_DRAWER(LessEquation);
	SET_EQUATION_DRAWER(MatrixEquation);
	SET_EQUATION_DRAWER(MinusEquation);
	SET_EQUATION_DRAWER(NoOperatorConnection);
	SET_EQUATION_DRAWER(OperatorConnection);
	SET_EQUATION_DRAWER(ParamEquation);
	SET_EQUATION_DRAWER(PlusEquation);
	SET_EQUATION_DRAWER(PowerEquation);
	SET_EQUATION_DRAWER(SubScriptEquation);
	SET_EQUATION_DRAWER(SummationEquation);
	SET_EQUATION_DRAWER(VectorEquation);
	SET_EQUATION_DRAWER(HatEquation);
	SET_EQUATION_DRAWER(TildeEquation);
	SET_EQUATION_DRAWER(FractionEquation);
	SET_EQUATION_DRAWER(IntegralSymbolEquation);
	SET_EQUATION_DRAWER(ProductEquation);
	SET_EQUATION_DRAWER(ProductSymbolEquation);
	SET_EQUATION_DRAWER(SummationSymbolEquation);
	SET_EQUATION_DRAWER(ConditionEquation);
	SET_EQUATION_DRAWER(AlignmentEquation);
	SET_EQUATION_DRAWER(DotEquation);
	SET_EQUATION_DRAWER(AsteriskEquation);
	SET_EQUATION_DRAWER(PrimeEquation);
	SET_EQUATION_DRAWER(SpaceEquation);
	SET_EQUATION_DRAWER(NotImplementedEquation);
	SET_EQUATION_DRAWER(DefinitionEquation);
	SET_EQUATION_DRAWER(ReferenceEquation);
	SET_EQUATION_DRAWER(EquationEnumeratorEquation);
	SET_EQUATION_DRAWER(CasesEquation);
	SET_EQUATION_DRAWER(TaggedEquation);
	SET_EQUATION_DRAWER(LinkEquation);
	SET_EQUATION_DRAWER(ConditionValueConnection);
	SET_EQUATION_DRAWER(RawDescriptionEquation);
	SET_EQUATION_DRAWER(VectorParamEquation);
	SET_EQUATION_DRAWER2(NoNumberEquation, EquationDummyDCDrawer);

	SET_EQUATION_DRAWER(LimitEquation);
	SET_EQUATION_DRAWER(MaxEquation);
	SET_EQUATION_DRAWER(MinEquation);
	SET_EQUATION_DRAWER(InfEquation);
	SET_EQUATION_DRAWER(SupEquation);

	SET_EQUATION_DRAWER(LimitSymbolEquation);
	SET_EQUATION_DRAWER(MaxSymbolEquation);
	SET_EQUATION_DRAWER(MinSymbolEquation);
	SET_EQUATION_DRAWER(InfSymbolEquation);
	SET_EQUATION_DRAWER(SupSymbolEquation);


#define DEF_EQUATION(CLASS, STR, TEX_DESC, TEX_DRAWER, WIN_DRAWER) \
	SET_EQUATION_DRAWER2(CLASS,WIN_DRAWER)
#include "BackSlashSymbols.h"
#include "BiOPs.h"
#include "CompOPs.h"
#include "ContentEqs.h"
#include "ArrowSymbols.h"
#undef DEF_EQUATION

	checkEqs();
}


#define INIT_EQUATION_DRAWER(EQUATION) EQUATION::init()

void Math2DWinDrawer::freeDrawer()
{
	INIT_EQUATION_DRAWER(BracketEquation);
	INIT_EQUATION_DRAWER(EqualEquation);
	INIT_EQUATION_DRAWER(GreaterEquation);
	INIT_EQUATION_DRAWER(GreekCharacter);
	INIT_EQUATION_DRAWER(IntegralEquation);
	INIT_EQUATION_DRAWER(LessEquation);
	INIT_EQUATION_DRAWER(MatrixEquation);
	INIT_EQUATION_DRAWER(MinusEquation);
	INIT_EQUATION_DRAWER(NoOperatorConnection);
	INIT_EQUATION_DRAWER(OperatorConnection);
	INIT_EQUATION_DRAWER(ParamEquation);
	INIT_EQUATION_DRAWER(PlusEquation);
	INIT_EQUATION_DRAWER(PowerEquation);
	INIT_EQUATION_DRAWER(SubScriptEquation);
	INIT_EQUATION_DRAWER(SummationEquation);
	INIT_EQUATION_DRAWER(VectorEquation);
	INIT_EQUATION_DRAWER(HatEquation);
	INIT_EQUATION_DRAWER(TildeEquation);
	INIT_EQUATION_DRAWER(FractionEquation);
	INIT_EQUATION_DRAWER(IntegralSymbolEquation);
	INIT_EQUATION_DRAWER(ProductEquation);
	INIT_EQUATION_DRAWER(ProductSymbolEquation);
	INIT_EQUATION_DRAWER(SummationSymbolEquation);
	INIT_EQUATION_DRAWER(LimitSymbolEquation);
	INIT_EQUATION_DRAWER(LimitEquation);
	INIT_EQUATION_DRAWER(ConditionEquation);
	INIT_EQUATION_DRAWER(AlignmentEquation);
	INIT_EQUATION_DRAWER(DotEquation);
	INIT_EQUATION_DRAWER(AsteriskEquation);
	INIT_EQUATION_DRAWER(PrimeEquation);
	INIT_EQUATION_DRAWER(SpaceEquation);
	INIT_EQUATION_DRAWER(NotImplementedEquation);
	INIT_EQUATION_DRAWER(DefinitionEquation);
	INIT_EQUATION_DRAWER(ReferenceEquation);
	INIT_EQUATION_DRAWER(EquationEnumeratorEquation);
	INIT_EQUATION_DRAWER(CasesEquation);
	INIT_EQUATION_DRAWER(TaggedEquation);
	INIT_EQUATION_DRAWER(LinkEquation);
	INIT_EQUATION_DRAWER(ConditionValueConnection);
	INIT_EQUATION_DRAWER(RawDescriptionEquation);


#define DEF_EQUATION(EQ, STR, TEX_DESC, TEX, WIN) INIT_EQUATION_DRAWER(EQ)
#include "BackSlashSymbols.h"
#include "BiOPs.h"
#include "CompOPs.h"
#include "ContentEqs.h"
#include "ArrowSymbols.h"
#undef DEF_EQUATION

	for (int i=0; i<s_drawerCnt; i++){
		EquationDrawer **p = &s_drawers[i];
		delete *p;
		*p = 0;
	}
	s_drawerCnt = 0;
}
