/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Math2DWinDrawer.h,v 1.23 2003/10/02 10:19:49 yoshi Exp $ */ 
#ifndef Math2DWinDrawer_h
#define Math2DWinDrawer_h

#include <windows.h>
#include "ErrorTextWriter.h"
#include "TagEquationManager.h"
#include "EquationRegister.h"
#include "LinkEquationManager.h"
#include "TagViewBufferManager.h"
#include "EquationDictionary.h"
#include "Math2DLoader.h"
#include "Equation.h"

class ViewBuffer;
class Length;
class Math2DContextCreatorBase;

class Math2DWinDrawer :
	public ErrorTextWriter,
	public EquationRegister
{
public:
	class ContentUpdateListener {
	public:
		virtual ~ContentUpdateListener() {;}
		virtual void updated() = 0;
		virtual void resize(Length &w, Length &h) = 0;
	};

private:
	ContentUpdateListener	*	m_updateListener;
	ErrorTextWriter		*	m_errWriter;
private:
	Math2DLoader	*	m_loader;
	ViewBuffer	*	m_origBuffer;
	ViewBuffer	*	m_buffer;
	
	TagEquationManager	m_taggedMap;
	LinkEquationManager	m_linkMgr;
	TagViewBufferManager	m_bufferMgr;

	Math2DContextCreatorBase &	m_contextCreator;

	EquationDictionary	m_dictionary;
	DrawContext *		m_context;

private:
	void	init(HDC, int width, int height);
	void	free_();

private:
	void	write(const char *, int);

	Equation::Size &calcSize(Math2DLoader::EquationContainer &);
	void	redraw(ViewBuffer *newbuffer)
	{
		m_buffer = newbuffer;
		if (m_updateListener) { m_updateListener->updated(); }
	}
	void	setEquation(TaggedEquation *, DrawContext *);
	void	setEquation(LinkEquation *, DrawContext *);

	DrawContext * drawContext();

	void	setErrorWriter(Math2DLoader *loader);
	ViewBuffer* checkHilight(char *tags);
public:
	Math2DWinDrawer(HDC hDC, int width, int height, Math2DContextCreatorBase &);
	virtual ~Math2DWinDrawer() { free_(); }

	// gpOEgpɌĂяoB
	static void	initDrawer();
	static void	freeDrawer();

	void	setErrorTextWriter(ErrorTextWriter *p) { m_errWriter = p; }
	void	setContentUpdateListener(ContentUpdateListener *p) { m_updateListener = p; }	
	void	resizeBuffer(HDC hDC, int width, int height) { init(hDC, width, height); }

	bool	draw(const char *filename);
	
	ViewBuffer *	getBuffer() { return m_buffer; }
	void	hilight(char tag);
	void	hilight(const char * tags);
	bool	hilighting() { return (m_buffer != m_origBuffer)? true: false; }
	char *	inLinkRect(int x, int y);
	void	clear();

};

#endif  // Math2DWinDrawer_h
