/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: StringMap.h,v 1.4 2002/09/26 08:51:57 yoshi Exp $ */ 
#ifndef StringMap_h
#define StringMap_h

#include "CharString.h"

#ifndef NO_STL

#include "List.h"
#include "Map.h"

// STL gp}bv
template <class T>
class StringMap : public Map<CharString, T, CharStringLess>
{
public:
	typedef CharString Key;
private:
	typedef Map<CharString, T, CharStringLess> Super;
	typedef List<Key> KeyContainer;
	typedef ListIterator<Key> KeyContainerIterator;
private:
	KeyContainer	m_keys;
private:
	void	free_()
	{
		KeyContainerIterator itr(&m_keys);
		while (itr.hasMoreElements()) {
			Key key = itr.element();
			free(key);
		}
		m_keys.clear();
		Super::clear();
		
	}
public:
	~StringMap() { free_(); }

	void	set(Key key, T t)
	{
		char *dup = strdup(key);
		Super::set(dup, t);
		m_keys.pushBack(dup);
	}

	void	clear() { free_(); }
};

#else // NO_STL

#include <stdlib.h>
#include "schash.h"

template<class T>
class StringMap
{
public:
	typedef CharString Key;
private:
	schash	*m_hash;

private:
	schash	*hash() {
		if (!m_hash) { m_hash = schash_new(5); }
		return m_hash;
	}
	void	free_()
	{
		schash *h = hash();
		if (!h) { return; }
		kvunit *curr = schash_first(h);
		while (curr) {
			char *key = const_cast<char*>(curr->key); free(key);
			curr = schash_next(h);
		}
		schash_delete(h);
		m_hash = 0;
	}
public:
	StringMap() : m_hash(0) {;}
	~StringMap() { free_(); }

	void	set(Key key, T t)
	{
		schash *h = hash();
		kvunit *ku = schash_find(h, key);
		if (ku) {
			ku->val.p = reinterpret_cast<void*>(t);
		} else {
			char *keydup = strdup(key);
			schash_insertp(h, keydup, (void*)t, 0);
		}
	}

	T	get(Key key)
	{
		kvunit *ku = schash_find(hash(), key);
		if (!ku) { return 0; }
		return reinterpret_cast<T>(ku->val.p);
	}

	void	clear() { free_(); }
};

#endif // NO_STL

#endif // StringMap_h
