/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: AlignEquationParser.cpp,v 1.3 2002/09/11 05:26:16 yoshi Exp $ */ 
#include "AlignEquationParser.h"
#include "AlignmentEquation.h"
#include "QuotationPair.h"
#include "EquationParseCmpnt.h"

const char AMP_CHAR = '&';

bool AlignEquationParser::match(char *str, int head)
{
	char *curr = str + head;
	return (*curr == AMP_CHAR)? true: false;
}

template<char C>
class CharQuatationPair : public QuotationPair
{
private:
	int	startLength() { return 1; }
	int	endLength() { return 1; }
	char *start(char *str,  int head)
	{
		char *curr = str + head;
		return (*curr == C)? curr: 0;
	}

	char *end(char *str,  int head)
	{
		char *curr = str + head;
		return (*curr == C)? curr: 0;
	}
};

typedef CharQuatationPair<AMP_CHAR> AlignmentQuatationPair;

Equation *AlignEquationParser::
parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	int	sx = itr.currx();
	char *	buf = itr.lineString();
	assert(*buf == AMP_CHAR);

	AlignmentQuatationPair alignPair;
	QuotationPair *pair = &alignPair;

	char *p;
	for (p=buf+1; *p; p++){
		if (pair->end(buf, p-buf)) { break; }
	}
	assert(*p);
	int ex = sx + p - buf;

	AlignmentEquation *align = new AlignmentEquation;

	ParseRange *		range;

	int	sy = target->top();
	int	ey = target->bottom();

	ScriptBlockWalker *block = itr.block();
	EquationRangeParserBase *rparser = createRangeParser();
	// left
	try {
		range = new ParseRange(target->left(), sy, sx-1, ey);
		EquationParseCmpnt cmpnt(block, range);
		Equation *eq = rparser->parse(&cmpnt);
		if (eq) { align->left(eq); }
	} catch(ParseRange::OutOfRange ) {;}

	// middle
	try {
		range = new ParseRange(sx + 1, sy, ex - 1, ey);
		EquationParseCmpnt cmpnt(block, range);
		Equation *eq = rparser->parse(&cmpnt);
		if (eq) { align->middle(eq); }
	} catch(ParseRange::OutOfRange ) {;}

	// right
	try {
		range = new ParseRange(ex + 1, sy, target->right(), ey);
		EquationParseCmpnt cmpnt(block, range);
		Equation *eq = rparser->parse(&cmpnt);
		if (eq) { align->right(eq); }
	} catch(ParseRange::OutOfRange ) {;}

	if (parsed) { *parsed = *target; }

	delete rparser;

	return align;
}

