/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: BracketEquationParser.cpp,v 1.15 2003/11/04 10:57:59 yoshi Exp $ */ 
#include "BracketEquationParser.h"
#include "EquationParseCmpnt.h"
#include "BracketPairParserManager.h"
#include "BracketPairParser.h"
#include "BracketEquation.h"
#include "QuotationEliminator.h"
#include "StringUtil.h"
#include "ParserException.h"

bool BracketEquationParser::match(char *str, int head)
{
	return (BracketPairParserManager::startBracket(&str[head]))? true: false;
}

class BracketMatcher : public QuotationEliminator::Matcher
{
private:
	BracketPair	*m_pair;
public:
	BracketMatcher(BracketPair *p) : QuotationEliminator::Matcher(), m_pair(p) {;}
	char* match(char *str, int head) { return m_pair->end(str, head); }
};

inline char *skipBlank(char *str) { return StringUtil::skip(str, SCRIPT_BLOCK_ELEMENT_BLANK); }

Equation * BracketEquationParser::
parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	int	sx = itr.currx();
	char *	buf = itr.lineString();
	if (!buf) { return 0; }
	
	BracketPairParser *bparser = BracketPairParserManager::startBracketCreate(buf);
	assert(bparser);

	int	start_len = bparser->startLength();

	BracketMatcher matcher(bparser);
	QuotationEliminator elim; elim.set(&matcher);
	char *find = elim.find(buf+start_len);

	BracketEquation *bracket = 0;
	ParseRange	*range;
	int		ex;
	if (find) {
		ex = sx + (find - buf);
		bracket = bparser->createEquation();
		int	end_len = bparser->endLength();
		try {
			range = new ParseRange(sx+start_len, target->top(),
					       ex-1, target->bottom());
		} catch (ParseRange::OutOfRange) { assert(0); }
	} else {
		int sy = itr.curry();
		do {
			if (!itr.moveDown()) { break; }
			itr.setPosition(sx, itr.curry());
			buf = itr.lineString();
			find = elim.find(buf + start_len);
		} while(!find);

		if (!find) {
			itr.setPosition(sx, sy);
			DTHROW_PE_MESSAGE(ParserIllegalDescriptionException, itr.totalLineIndex(),
			       ("Bracket not match"));
		}

		ex = sx + (find - buf);
		bracket = bparser->createEquation();
		int	end_len = bparser->endLength();
		try {
			range = new ParseRange(sx+start_len, sy,
					       ex-1, itr.curry());
		} catch(ParseRange::OutOfRange) { assert(0); }
	}

	assert(bracket);

	EquationParseCmpnt cmpnt(itr.block(), range);
	EquationRangeParserBase *rparser = createRangeParser();
	Equation *eq = rparser->parse(&cmpnt);
	if (eq) { bracket->content(eq); }
	
	if (parsed) {
		parsed->set(sx, target->top(), ex+bparser->endLength()-1, target->bottom());
	}

	delete bparser;
	delete rparser;
	return bracket;
}

	



