/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: BracketPairParserManager.cpp,v 1.13 2003/12/15 13:24:02 yoshi Exp $ */
#include "BracketPairParserManager.h"
#include "BracketPairParser.h"
#include "BracketEquation.h"
#include <ctype.h>

const char DOT_CHAR = '.';
const char BACKSLASH_CHAR = '\\';

static bool bracketDotTerminateion(char *p, bool seePrev)
{
	// '.' AĂ
	if ( (seePrev && (*(p-1) == DOT_CHAR)) ||
	     (*(p+1) == DOT_CHAR)) { return false; }
	else if ( !isalnum(*(p+1)) ) {  // ܂̓At@xbgł͂Ȃ
		return true;
	}
	return false;
}

class BracketStrParser
{
private:
	int	m_length;
	bool	m_visible;
	BracketEquation::Shape m_shape;
public:
	BracketStrParser()
		: m_length(0), m_visible(true), m_shape(BracketEquation::SHAPE_NOT_SET) {;}
	char *start(char *p, int head);
	char *end(char *p, int head);

	bool	visible() const { return m_visible; }
	int	length() const { return m_length; }
	BracketEquation::Shape shape() const { return m_shape; }
};

#define STRING_EQUALS(BUF, STR) (strncmp(BUF, STR, strlen(STR)) == 0)

#define FLOOR_STR "floor"
#define CEIL_STR "ceil"
#define ANGLE_STR "angle"

char *BracketStrParser::start(char *str, int head)
{
	char *h = str + head;
	char *p = h;
	if (!*p) { return 0; }

	bool bs = false;
	if (*p == BACKSLASH_CHAR) {
		bs = true;
		p++;
	}

	switch(*p) {
	case '(': m_shape = BracketEquation::SHAPE_ROUND; p++; break;
	case '{':
		m_shape = BracketEquation::SHAPE_BRACE;
		if (!bs) { m_visible = false; }
		p++;
		break;
	case '[': m_shape = BracketEquation::SHAPE_SQUARE; p++; break;
	case 'l':
		if (!bs) { return 0; }
		p++;
		if (STRING_EQUALS(p, FLOOR_STR)) {
			m_shape = BracketEquation::SHAPE_FLOOR; p += strlen(FLOOR_STR);
		} else if (STRING_EQUALS(p, CEIL_STR)) {
			m_shape = BracketEquation::SHAPE_CEIL;  p += strlen(CEIL_STR);
		} else if (STRING_EQUALS(p, ANGLE_STR)) {
			m_shape = BracketEquation::SHAPE_ANGLE; p += strlen(ANGLE_STR);
		} else { return 0; }
		break;
	default:
		return 0;
	}

	m_length = (p-h);
	return (h);
}

char *BracketStrParser::end(char *str, int head)
{
	char *h = str + head;
	char *p = h;
	if (!*p) { return 0; }

	bool	bs = false;
	if (*p == DOT_CHAR) {
		if (bracketDotTerminateion(p, (head > 0)? true: false)) { 
			m_shape = BracketEquation::SHAPE_NOT_SET;
			m_length = 1;
			return h;
		} else { return 0; }
	} else if (*p == BACKSLASH_CHAR) {
		bs = true;
		p++;
	} 

	switch(*p) {
	case ']': m_shape = BracketEquation::SHAPE_SQUARE; p++; break;
	case '}':
		m_shape = BracketEquation::SHAPE_BRACE; p++;
		if (!bs) { m_visible = false; }
		break;
	case ')': m_shape = BracketEquation::SHAPE_ROUND; p++; break;
	case 'r':
		if (!bs) { return 0; }
		p++;
		if (STRING_EQUALS(p, FLOOR_STR)) {
			m_shape = BracketEquation::SHAPE_FLOOR; p += strlen(FLOOR_STR);
		} else if (STRING_EQUALS(p, CEIL_STR)) {
			m_shape = BracketEquation::SHAPE_CEIL; p += strlen(CEIL_STR); 
		} else if (STRING_EQUALS(p, ANGLE_STR)) {
			m_shape = BracketEquation::SHAPE_ANGLE; p += strlen(ANGLE_STR);
		} else { return 0; }
		break;
		
	default:
		return 0;
	}
	m_length = (p-h);
	return (h);
}

class BracketPairParserImpl : public BracketPairParser
{
private:
	BracketStrParser	m_start;
	BracketStrParser	m_end;

private:
	char * start(char *str, int head) { return m_start.start(str, head); }
	char * end(char *str, int head) { return m_end.end(str, head); }
	int	startLength() { return m_start.length(); }
	int	endLength() { return m_end.length(); }
	BracketEquation *createEquation()
	{
		return new BracketEquation(m_start.shape(), m_end.shape(),
					   m_start.visible(), m_end.visible());
	}
};


class BracketPairParserHolder
{
private:
	enum { PARSER_SIZE = 1, };
	BracketPairParser *m_brackets[PARSER_SIZE];
private:
	void	free_()
	{
		for (int i=0; i<PARSER_SIZE; i++) {
			BracketPairParser **p = &m_brackets[i];
			if (*p) { delete *p; *p = 0; }
		}
	}
			     
			
public:
	BracketPairParserHolder()
	{
		int	i = 0;
		m_brackets[i] = new BracketPairParserImpl; i++;
		assert(i == PARSER_SIZE);
	}
	~BracketPairParserHolder() { free_(); }

	BracketPairParser *element(int i) { return m_brackets[i]; }
	int	size() { return PARSER_SIZE;}
};

#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))

static BracketPairParserHolder holder;

bool BracketPairParserManager::Iterator::hasMoreElements()
{
	if (m_curr < 0) { m_curr = 0; }
	else { m_curr++; }
	if (m_curr >= 1) { m_curr = -1; return false; }
	return true;
}


BracketPairParser *BracketPairParserManager::Iterator::element()
{
	return new BracketPairParserImpl;
}

BracketPairParser *BracketPairParserManager::startBracketCreate(char *str)
{
	BracketPairParser* p = new BracketPairParserImpl;
	if (p->start(str, 0)) { return p; }
	delete p;
	return 0;
}

bool BracketPairParserManager::startBracket(char *str)
{
	BracketPairParser* p = new BracketPairParserImpl;
	bool b = (p->start(str, 0))? true: false;
	delete p;
	return b;
}
