/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationRangeParser.cpp,v 1.8 2004/03/25 07:21:33 yoshi Exp $ */ 
#include <assert.h>
#include "EquationRangeParser.h"
#include "EquationParserFactory.h"
#include "EquationParseCmpnt.h"
#include "ScriptBlockWalker.h"
#include "ScriptBlockWalkerUtil.h"
#include "EquationParser.h"
#include "SuffixRangeExtractor.h"
#include "SubScriptEquation.h"
#include "PowerEquation.h"
#include "NoOperatorConnection.h"
#include "ConditionValueConnection.h"
#include "ScriptBlockRangeIterator.h"

inline int baseline(ScriptBlockWalker *block, ParseRange *range)
{
	return ScriptBlockWalkerUtil::baseline(block,
					       range->left(), range->top(),
					       range->right(), range->bottom());
}

// p[XIO
// p[X͈͂
class ParseEnd : public ParseRange
{
public:
	ParseEnd(const ParseRange &o) : ParseRange(o) {;}
	ParseEnd(int l, int t, int r, int b) : ParseRange(l, t, r, b) {;}
};

Equation *EquationRangeParser::parseNewLine(EquationParseCmpnt *cmpnt, ParseRange &parsed)
{
	assert(cmpnt);
	assert(m_factory);
	Equation *whole = 0;
	ParseRange *target;
	try { target = new ParseRange(cmpnt->range()); }
	catch (ParseRange::OutOfRange) { return 0; }

	ScriptBlockWalker *block = cmpnt->block();
	EquationParser *parser = 0;
	try {
		int base = cmpnt->baseline();
		if (base == EquationParseCmpnt::BASELINE_NOT_SET) {
			base = baseline(cmpnt->block(), target);
			if (base < 0) { throw ParseEnd(*target); } 
		}

		int sx;
		parser = m_factory->createNewLine(block, target, base, sx);
		if (parser) {
			whole = parser->parseBaseline(cmpnt->block(), target, base, sx, &parsed);
			if (!whole) {  throw ParseEnd(*target); }
		}

	} catch (ParseEnd &e) {
		parsed = e;
	}

	if (parser) { EquationParserFactory::free(parser); }
	delete target;

	return whole;
}


Equation *EquationRangeParser::parse(EquationParseCmpnt *cmpnt, ParseRange &parsed)
{
	assert(cmpnt);
	assert(m_factory);
	Equation *whole = 0;
	ParseRange *target;
	try { target = new ParseRange(cmpnt->range()); }
	catch (ParseRange::OutOfRange) { return 0; }

	ScriptBlockWalker *block = cmpnt->block();
	EquationParser *parser = 0;
	try {
		int base = cmpnt->baseline();
		if (base == EquationParseCmpnt::BASELINE_NOT_SET) {
			base = baseline(cmpnt->block(), target);
			if (base < 0) { throw ParseEnd(*target); } 
		}

		int	sx;
		parser = m_factory->create(block, target, base, sx);
		if (!parser) { throw ParseEnd(*target); }
		whole = parser->parseBaseline(cmpnt->block(), target, base, sx, &parsed);
		if (!whole) {  throw ParseEnd(*target); }

		// x[XC̎̋LqJnʒu
		BlockRange &range = cmpnt->range();
		ScriptBlockRangeIterator itr(cmpnt->block(), &range);
		itr.setPosition(parsed.right()+1, base);
		itr.skipBlankRight();
		int next_start_x = itr.currx();

		// Ÿ̐؂o
		SuffixRangeExtractor extractSuffix;
		if (extractSuffix(block, base, 
				  parsed.right()+1, next_start_x-1,
				  target->top(), target->bottom())) {
			EquationParseCmpnt *cmpnt = 0;
			if (cmpnt = extractSuffix.subCmpnt()) { 
				Equation *eq = parse(cmpnt);
				if (eq) {
					SubScriptEquation *sub = new SubScriptEquation;
					sub->base(whole); sub->suffix(eq);
					whole = sub;
				}
			}
			if (cmpnt = extractSuffix.powCmpnt()) {
				Equation *eq = parse(cmpnt);
				if (eq) {
					PowerEquation *pow = new PowerEquation;
					pow->base(whole); pow->suffix(eq);
					whole = pow;
				}
			}
		}

		// Yp[Ẍ̍XV
		parsed.right(next_start_x-1);

	} catch (ParseEnd &e) {
		parsed = e;
	}
		
	if (parser) { EquationParserFactory::free(parser); }
	delete target;
	return whole;
}

static Equation *connect(Equation *left, Equation *right)
{
	if (right->isCondition()) {
		return new ConditionValueConnection(left, right);
	} else {
		return new NoOperatorConnection(left, right);
	}
	assert(0);
	return 0;
}


Equation *EquationRangeParser::parse(EquationParseCmpnt *cmpnt)
{
	Equation *whole = parse(cmpnt, m_parsed);

	if (!cmpnt->single()) { 
		ParseRange &target = cmpnt->range();
		try {
			// E̋Lq̐؂o
			ParseRange *range = new ParseRange(m_parsed.right()+1, target.top(),
							   target.right(), target.bottom());
			EquationParseCmpnt cmpnt(cmpnt->block(), range);
			ParseRange parsed;
			Equation *eq = parse(&cmpnt);
			if (eq) {
				whole = connect(whole, eq);
			}
			m_parsed.right(parsed.right());

		} catch(ParseRange::OutOfRange) {;}
	}

	return whole;
}

