/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: MatrixEquationParser.cpp,v 1.4 2002/08/14 09:14:24 yoshi Exp $ */
#include "MatrixEquationParser.h"
#include "EquationParseCmpnt.h"
#include "MatrixEquation.h"
#include "CasesEquation.h"
#include <memory.h>

#define MATRIX_ELEMENT_SPLIT_CHAR '#'

bool MatrixEquationParser::match(char *str, int head)
{
	const char curr = str[head];
	return (curr == MATRIX_ELEMENT_SPLIT_CHAR)? true: false;
}

static int nextRowStart(ScriptBlockIterator &itr)
{
	while (*itr == MATRIX_ELEMENT_SPLIT_CHAR) {
		if (!itr.moveDown()) { return -1; }
	}
	return itr.curry();
}

static int rowEnd(ScriptBlockIterator &itr)
{
	while (*itr != MATRIX_ELEMENT_SPLIT_CHAR) {
		if (!itr.moveDown()) { return -1; }
	}
	itr.moveUp();
	return itr.curry();
}

static int nextColumnStart(ScriptBlockIterator &itr)
{
	if (!itr.right()) { return -1; }
	while (*itr == MATRIX_ELEMENT_SPLIT_CHAR) { 
		if (!itr.moveRight()) { return -1; }
	}
	return itr.currx();
}

static int columnEnd(ScriptBlockIterator &itr)
{
	while (*itr != MATRIX_ELEMENT_SPLIT_CHAR) {
		if (!itr.moveRight()) { return itr.currx(); }
	}
	itr.moveLeft();
	return itr.currx();
}

#define MAX(a, b) ( ((a) > (b))? (a): (b))


Equation *MatrixEquationParser::
parse(ScriptBlockIterator &itr, ParseRange *target, ParseRange *parsed)
{
	const int BUFFER_SIZE = 32;
	const int COL_SIZE = BUFFER_SIZE;
	const int ROW_SIZE = BUFFER_SIZE;

	static Equation *eqs[COL_SIZE][ROW_SIZE];
	memset(eqs, 0x00, sizeof(eqs));

	ScriptBlockWalker *block = itr.block();
	EquationRangeParserBase *rparser = createRangeParser();

	int startx = itr.currx();

	bool cases = false;
	int rowSize = 0;
	int colSize = 0;

	int row;
	for (row=0; ;row++) {
		int y = itr.curry();
		itr.setPosition(startx, y);

		int sy = nextRowStart(itr); if (sy < 0) { break; }
		int ey = rowEnd(itr); if (ey < 0) { break; }
		int col;
		for (col=0; ;col++) {
			int sx = nextColumnStart(itr); if (sx < 0) { break; }
			int ex = columnEnd(itr); if (ex < 0) { break; }
			try {
				ParseRange *range = new ParseRange(sx, sy, ex, ey);
				EquationParseCmpnt cmpnt(block, range);
				Equation *eq = rparser->parse(&cmpnt);
				if (eq) {
					if (eq->containCondition()) {
						cases = true;
					}
					eqs[col][row] = eq;
				}
			} catch (ParseRange::OutOfRange) {;}
			itr.moveRight();
		}
		colSize = MAX(col, colSize);
		itr.moveDown();
	}
	rowSize = row;

	delete rparser;
	if (parsed) { *parsed = *target; }

	MatrixStructuredEquation *mat = (cases)?
		static_cast<MatrixStructuredEquation*>(new CasesEquation):
		static_cast<MatrixStructuredEquation*>(new MatrixEquation);
	for (row=0; row<rowSize; row++) {
		for (int col=0; col<colSize; col++){
			mat->element(col, row, eqs[col][row]);
		}
	}

	return mat;
}

