/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: StringBlockBuilder.cpp,v 1.5 2003/10/02 10:10:52 yoshi Exp $ */
#include "StringBlockBuilder.h"
#include "TextLineReadIterator.h"
#include "StringHLine.h"
#include "StringBlockUnit.h"
#include "StringBlock.h"
#include "LineChecker.h"

class Distributor
{
private:
	StringBlockUnit	*		m_unit;
	StringBlock	*		m_block;
	StringBlockBuilder::Container *	m_container;

	int				m_line;

private:	
	StringBlockUnit	*  unit() { return m_unit; }
	StringBlock	*  block() { return m_block; }

	StringBlockUnit *  createUnit() { return (m_unit = new StringBlockUnit); }
	StringBlock *	createBlock() { return (m_block = new StringBlock); }
		
	void	popUnit();
	void	popBlock();

	void	postProcess() { popBlock(); }
public:
	Distributor() : m_unit(0), m_block(0), m_container(0) {;}
	~Distributor() { postProcess(); }
	
	void	set(StringBlockBuilder::Container *p) { m_container = p; }
	void	set(char *str, LineChecker::NormalLine *normal);
	void	set(LineChecker::SplitBlock *p) { popBlock(); }
	void	set(LineChecker::SplitUnit *p) { popUnit(); }
	void	line(int n) { m_line = n; }
};

void	Distributor::popUnit()
{
	if (!m_unit) { return; }
	if (!block()) { createBlock(); }
	block()->pushBack(m_unit); m_unit = 0;
}

void	Distributor::popBlock()
{
	if (!m_container) { return; }
	if (unit()) { popUnit(); }
	if (m_block) { m_container->pushBack(m_block);  m_block = 0;}
}

void Distributor::set(char *str, LineChecker::NormalLine *normal)
{
	StringHLine *line = new StringHLine(str, normal->start, m_line);
	if (!unit()) { createUnit(); }
	unit()->pushBack(line);
}



StringBlockBuilder::Container &
StringBlockBuilder::build(StringBlockBuilder::Container &container)
{
	if (!m_lineItr) { return container; }

	typedef LineChecker::NormalLine NormalLine;
	typedef LineChecker::SplitBlock SplitBlock;
	typedef LineChecker::SplitUnit SplitUnit;

	Distributor	dist;
	dist.set(&container);

	LineChecker	check;
	NormalLine *	normal;
	SplitBlock *	splitblock;
	SplitUnit *	splitunit;	

	while (m_lineItr->hasMoreElements()) {
		char *line = m_lineItr->element();
		dist.line(m_lineItr->line());

		LineChecker::Type type = check(line);
		switch(type) {
		case LineChecker::NORMAL_LINE:
			normal = reinterpret_cast<NormalLine*>(&check.result());
			dist.set(m_lineItr->releaseElement(), normal);
			break;
		case LineChecker::SPLIT_BLOCK:
			splitblock = reinterpret_cast<SplitBlock*>(&check.result());
			dist.set(splitblock);
			break;
		case LineChecker::SPLIT_UNIT:
			splitunit = reinterpret_cast<SplitUnit*>(&check.result());
			dist.set(splitunit);
			break;
		default:
			break;
		}
	}

	return container;
}

void StringBlockBuilder::Container::free_()
{
	ContainerIterator itr;
	itr.set(this);
	while(itr.hasMoreElements()) {
		StringBlock *block = itr.element();
		delete block;
	}
}

#ifdef DEBUG_StringBlockBuilder1

#include "TextStdinLineReadIterator.h"


int main()
{
	TextStdinLineReadIterator ritr;

	StringBlockBuilder	build;
	build.set(&ritr);
	StringBlockBuilder::Container container;
	build(container);

	StringBlockBuilder::ContainerIterator citr;
	citr.set(&container);
	while(citr.hasMoreElements()) {
		StringBlock *block = citr.element();
		print(block);
	}

	StringBlockBuilder::free(container);
}

#endif

#ifdef DEBUG_StringBlockBuilder2

#include "TextStdinLineReadIterator.h"
#include "StringBlockWalker.h"


int main()
{
	TextStdinLineReadIterator ritr;

	StringBlockBuilder	build;
	build.set(&ritr);
	StringBlockBuilder::Container container;
	build(container);

	StringBlockBuilder::ContainerIterator citr;
	citr.set(&container);
	while(citr.hasMoreElements()) {
		printf("block 1 : \n");
		StringBlock *block = citr.element();
		StringBlockWalker walker;
		walker.set(block);
		printScriptWalker(&walker);
	}
	StringBlockBuilder::free(container);
}

#endif

