/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: StringBlockUnit.cpp,v 1.2 2001/10/18 06:15:00 yoshi Exp $ */
#include "StringBlockUnit.h"
#include "IndexConverter.h"

const int CHAR_START_MAX = 10000;

int int_max(int n1, int n2)
{
	return (n1>n2)? n1: n2;
}


StringBlockUnit::
StringBlockUnit() : Array<StringHLine*>(128)
{
	m_width = 0;
	m_baseline = -1;
	m_charStart = CHAR_START_MAX;
	m_yConverter = 0;
}

void	StringBlockUnit::pushBack(StringHLine *line)
{
	Parent::pushBack(line);
	m_width = int_max(line->width(), m_width);
	if (line->charStart() < m_charStart) {
		m_charStart = line->charStart();
		m_baseline = size() - 1;
	}
}

StringHLine *StringBlockUnit::line(int blockY)
{
	int	unitY = yIndex(blockY);
	if (unitY < 0 || unitY >= height()) { return 0; }
	return Parent::element(unitY);
}



void StringBlockUnit::free()
{
	ThisIterator itr;
	itr.set(this);
	while (itr.hasMoreElements())
	{
		StringHLine *line = itr.element();
		delete line;
	}
}
/*
int  StringBlockUnit::xIndex(int i)
{
	return (m_xConverter)? m_xConverter->to(this, i): i;
}
*/

int  StringBlockUnit::yIndex(int i)
{
	return (m_yConverter)? m_yConverter->to(this, i): i;
}

int StringBlockUnit::upperBoundOffset()
{
	return -baseline();
}

int StringBlockUnit::lowerBoundOffset()
{
	return (height() - 1 - baseline());
}


#ifdef _DEBUG

#include <stdio.h>

void 	print(StringBlockUnit *unit)
{
	StringBlockUnit::ThisIterator itr;
	itr.set(unit);

	printf(".................................................\n");
	printf("unit : \n");
	printf("12345678901234567890123456789012345678901234567890\n");
	printf(".........1.........2.........3.........4.........5\n");

	int	cnt = 0;
	while(itr.hasMoreElements()) {
		StringHLine *line = itr.element();
		printf("%s : %d", line->buffer(), unit->yIndex(cnt));
		if (unit->baseline() == cnt) {
			printf("  <- baseline");
		}
		printf("\n");
		cnt++;
	}
}


#endif

#ifdef DEBUG_StringBlockUnit

#include <stdio.h>

int main()
{
	StringBlockUnit	unit;
	StringHLine *line;
	line = new StringHLine("abc", 0, false); unit.pushBack(line);
	line = new StringHLine("abcdef", 0, false); unit.pushBack(line);
       
	printf("width %d\n", unit.width());
	printf("height %d\n", unit.height());
	return 0;
}

#endif
