/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: DrawContext.cpp,v 1.10 2004/05/06 05:10:27 yoshi Exp $ */
#include "DrawContext.h"
#include <string.h>

DrawContext::DrawContext()
	: m_writer(0), m_counter(0), m_dictionary(0),
	  m_state(NEWLINE|RAW_DESCRIPTION|EQUATION_BLOCK), m_texType(TexType::NTT_JLATEX)
{
	;
}


DrawContext::DrawContext(const DrawContext &o)
	: m_writer(o.m_writer), m_counter(o.m_counter),
	  m_dictionary(o.m_dictionary), m_state(o.m_state), m_texType(o.m_texType)
{
	;
}

#define FREE(P) if (P) { delete P; P = 0; }

void DrawContext::free_()
{
	FREE(m_counter);
}

template<class C>
class CharReplacer
{
public:
	typedef C Converter;
private:
	C	m_converter;
public:
	char	*push(char *str)
	{
		for (char *p=str; *p; p++){
			if (m_converter.match(*p)) { *p = m_converter.to(); }
		}
		return str;
	}
	Converter	&converter() { return m_converter; }
};

class NewlineToSpace
{
public:
	bool	match(char c) { return (c == '\n')? true: false; }
	char	to() { return ' '; }
};

typedef CharReplacer<NewlineToSpace> NewlineReplacer;

class LineCountFilter : public DrawContext::LineCountWriter
{
private:
	DrawContext::Writer	*m_writer;
	bool	m_outputBlock;
public:
	LineCountFilter() :
		DrawContext::LineCountWriter() , m_writer(0), m_outputBlock(false) {;}
	void	set(DrawContext::Writer *p) { m_writer = p; }
	void	outputBlock(bool b) { m_outputBlock = b; }
	void	write(const char *str) { m_writer->write(str);	}
	void	write(char c) { m_writer->write(c); }
	int	calcLineNum(int line)
	{
		int l = line;
		if (m_outputBlock) { l++ ; }
		return l;
	}
};

class NewlineRemoveFilter : public LineCountFilter
{
private:
	typedef LineCountFilter Super;
public:
	typedef NewlineReplacer::Converter Converter;
private:
	NewlineReplacer	m_replacer;
	int		m_newline;

private:
	void write(const char *str)
	{
		char	buf[1024];
		strcpy(buf, str);
		Super::write(m_replacer.push(buf));
	}
	
	void write(char c)
	{
		NewlineRemoveFilter::Converter &converter = m_replacer.converter();
		char to = (converter.match(c))? converter.to(): c;
		Super::write(to);
	}
	int	line() { return calcLineNum(m_newline); }
	void	newline(const char *head) {
		if (head) { Super::write(head); }
		Super::write("\n"); m_newline++;
	}
public:
	NewlineRemoveFilter() : LineCountFilter(), m_newline(0) {;}
};

void DrawContext::set(Writer *writer)
{
	m_writer = writer;
}

class NewlineCounter : public LineCountFilter
{
private:
	typedef LineCountFilter Super;
private:
	int	m_line;
private:
	void	write(const char *str)
	{
		char *s = const_cast<char*>(str);
		for (char *p = s; *p; p++) {
			if (*p == '\n') { m_line++; }
		}
		Super::write(str);
	}
	void	write(char c)
	{
		if (c == '\n') { m_line++; }
		Super::write(c);
	}
	int	line() { return calcLineNum(m_line); }
	void	newline(const char *head)
	{
		if (head) { write(head); }
		write('\n');
	}
public:
	NewlineCounter() : Super(), m_line(0) {;}
};


void DrawContext::setup()
{
	LineCountFilter *p = 0;
	if (newline()) {
		p = new NewlineCounter;
	} else {
		p = new NewlineRemoveFilter;
	}
	const bool b = this->equationBlock();
	p->outputBlock(b);
	p->set(m_writer);
	m_counter = p;
}
	
