/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Math2DTexDrawer.cpp,v 1.25 2004/05/07 04:36:25 yoshi Exp $ */ 
#include "Math2DTexDrawer.h"
#include "EquationDrawer.h"
#include "Math2DLoader.h"
#include "DrawContext.h"
#include "eqcheck.h"
#include <memory.h>

Math2DTexDrawer::Math2DTexDrawer() : m_drawerCnt(0), m_loader(0)
{
	init();
}

void	Math2DTexDrawer::init()
{
	memset(m_drawers, 0x00, sizeof(m_drawers));
	initDrawer();
}

#define FREE(p) if (p) { delete p; p = 0; }

void	Math2DTexDrawer::free_()
{
	FREE(m_loader);
	int n = size();
	for (int i=0; i<n; i++) {
		EquationDrawer *p = m_drawers[i];
		delete p;
	}
}

Math2DLoader *Math2DTexDrawer::createLoader()
{
	FREE(m_loader);
	m_loader = new Math2DLoader;
	m_loader->set(m_err);
	return m_loader;
}

bool Math2DTexDrawer::load(const char *fname)
{
	Math2DLoader *loader = createLoader();
	return loader->load(fname);
}


bool Math2DTexDrawer::load(TextLineReadIterator &itr)
{
	Math2DLoader *loader = createLoader();
	return loader->load(itr);
}

static bool outputEquation(Equation *eq)
{
	return !eq->isDefinition();
}

void Math2DTexDrawer::draw(DrawContext *context)
{
	if (!m_loader) { return; }

	Math2DLoader::EquationContainer &eqs = m_loader->getContainer();
	Math2DLoader::EquationContainerIterator itr(&eqs);
	while (itr.hasMoreElements()) {
		Equation *eq = itr.element();
		if (eq) { eq->draw(context); }
	}
}


int Math2DTexDrawer::line()
{
	assert(m_loader);
	return m_loader->line();
}


#include "ParamEquation.h"
#include "OperatorConnection.h"
#include "PlusEquation.h"
#include "MinusEquation.h"
#include "FractionEquation.h"
#include "SummationEquation.h"
#include "ProductEquation.h"
#include "SubScriptEquation.h"
#include "PowerEquation.h"
#include "BracketEquation.h"
#include "NoOperatorConnection.h"
#include "MatrixEquation.h"
#include "VectorEquation.h"
#include "EqualEquation.h"
#include "AsteriskEquation.h"
#include "LessEquation.h"
#include "GreaterEquation.h"
#include "IntegralEquation.h"
#include "DotEquation.h"
#include "AlignmentEquation.h"
#include "HatEquation.h"
#include "PrimeEquation.h"
#include "TildeEquation.h"
#include "GreekCharacter.h"
#include "TaggedEquation.h"
#include "LinkEquation.h"
#include "SpaceEquation.h"
#include "ConditionEquation.h"
#include "DefinitionEquation.h"
#include "ReferenceEquation.h"
#include "EquationEnumeratorEquation.h"
#include "CasesEquation.h"
#include "ConditionValueConnection.h"
#include "RawDescriptionEquation.h"
#include "NotImplementedEquation.h"
#include "NoNumberEquation.h"
#include "VectorParamEquation.h"

#include "ArrowEquationImpl.h"
#include "ExtendedRangeEquationImpl.h"
#include "BackSlashEquationImpl.h"
#include "BinominalOperatorEquationImpl.h"
#include "ComparativeOperatorEquationImpl.h"
#include "ContentEquationImpl.h"


#include "BackSlashTexEquationDrawerImpl.h"
#include "BinominalOperatorEquationTexDrawer.h"
#include "ComparativeOperatorEquationTexDrawer.h"
#include "ParamEquationTexDrawer.h"
#include "OperatorConnectionTexDrawer.h"
#include "FractionEquationTexDrawer.h"
#include "SuperSubScriptEquationTexDrawerImpl.h"
#include "SubScriptEquationTexDrawer.h"
#include "PowerEquationTexDrawer.h"
#include "BracketEquationTexDrawer.h"
#include "NoOperatorConnectionTexDrawer.h"
#include "MatrixEquationTexDrawer.h"
#include "VectorEquationTexDrawer.h"
#include "IntegralEquationTexDrawer.h"
#include "DotEquationTexDrawer.h"
#include "AlignmentEquationTexDrawer.h"
#include "GreekCharacterTexDrawer.h"
#include "LinkEquationTexDrawer.h"
#include "TaggedEquationTexDrawer.h"
#include "SpaceEquationTexDrawer.h"
#include "ConditionEquationTexDrawer.h"
#include "DefinitionEquationTexDrawer.h"
#include "ReferenceEquationTexDrawer.h"
#include "EquationEnumeratorEquationTexDrawer.h"
#include "CasesEquationTexDrawer.h"
#include "ConditionValueConnectionTexDrawer.h"
#include "HatEquationTexDrawer.h"
#include "TildeEquationTexDrawer.h"
#include "RawDescriptionEquationTexDrawer.h"
#include "NotImplementedEquationTexDrawer.h"
#include "VectorParamEquationTexDrawer.h"

#include "SummationSymbolEquation.h"
#include "ProductSymbolEquation.h"
#include "IntegralSymbolEquation.h"

#include "DummyEquationTexDrawer.h"


#define SET_EQUATION_DRAWER(CLASS) \
	CLASS::setDrawer(m_drawers[m_drawerCnt++] = new CLASS##TexDrawer)

#define SET_EQUATION_DRAWER2(CLASS, DRAWER) \
	CLASS::setDrawer(m_drawers[m_drawerCnt++] = new DRAWER)


void	Math2DTexDrawer::initDrawer()
{
	SET_EQUATION_DRAWER(ParamEquation);
	SET_EQUATION_DRAWER(OperatorConnection);
	SET_EQUATION_DRAWER(PlusEquation);
	SET_EQUATION_DRAWER(MinusEquation);
	SET_EQUATION_DRAWER(FractionEquation);
	SET_EQUATION_DRAWER(SubScriptEquation);
	SET_EQUATION_DRAWER(PowerEquation);
	SET_EQUATION_DRAWER(BracketEquation);
	SET_EQUATION_DRAWER(NoOperatorConnection);
	SET_EQUATION_DRAWER(MatrixEquation);
	SET_EQUATION_DRAWER(VectorEquation);
	SET_EQUATION_DRAWER(AsteriskEquation);

	SET_EQUATION_DRAWER(EqualEquation);
	SET_EQUATION_DRAWER(LessEquation);
	SET_EQUATION_DRAWER(GreaterEquation);

	SET_EQUATION_DRAWER(IntegralEquation);
	SET_EQUATION_DRAWER(DotEquation);
	SET_EQUATION_DRAWER(AlignmentEquation);

	SET_EQUATION_DRAWER(HatEquation);

	SET_EQUATION_DRAWER(SummationEquation);
	SET_EQUATION_DRAWER(ProductEquation);
	SET_EQUATION_DRAWER(LimitEquation);
	SET_EQUATION_DRAWER(MaxEquation);
	SET_EQUATION_DRAWER(MinEquation);
	SET_EQUATION_DRAWER(SupEquation);
	SET_EQUATION_DRAWER(InfEquation);

	SET_EQUATION_DRAWER(PrimeEquation);
	SET_EQUATION_DRAWER(TildeEquation);
	SET_EQUATION_DRAWER(GreekCharacter);
	SET_EQUATION_DRAWER(LinkEquation);
	SET_EQUATION_DRAWER(TaggedEquation);
	SET_EQUATION_DRAWER(DeterminantEquation);
	SET_EQUATION_DRAWER(SpaceEquation);
	SET_EQUATION_DRAWER(ConditionEquation);
	SET_EQUATION_DRAWER(DefinitionEquation);
	SET_EQUATION_DRAWER(ReferenceEquation);
	SET_EQUATION_DRAWER(EquationEnumeratorEquation);
	SET_EQUATION_DRAWER(CasesEquation);
	SET_EQUATION_DRAWER(ConditionValueConnection);
	SET_EQUATION_DRAWER(RawDescriptionEquation);
	SET_EQUATION_DRAWER(NotImplementedEquation);
	SET_EQUATION_DRAWER2(NoNumberEquation, BackSlashTexEquationDrawer);
	SET_EQUATION_DRAWER(VectorParamEquation);

	SET_EQUATION_DRAWER2(SummationSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(ProductSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(IntegralSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(LimitSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(MaxSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(MinSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(SupSymbolEquation, DummyEquationTexDrawer);
	SET_EQUATION_DRAWER2(InfSymbolEquation, DummyEquationTexDrawer);


#define DEF_EQUATION(EQ, S, TEX_DESC, TEXDRAWER, WIN) SET_EQUATION_DRAWER2(EQ, TEXDRAWER)
#include "ArrowSymbols.h"
#include "BackSlashSymbols.h"
#include "CompOPs.h"
#include "BiOPs.h"
#include "ContentEqs.h"
#undef DEF_EQUATION

	checkEqs();
}
