/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTexArgsTokenizer.cpp,v 1.15 2004/05/07 04:36:51 yoshi Exp $ */ 
#include "PHTexArgsTokenizer.h"
#include "TextFileLineReadIterator.h"
#include <stdio.h>
#include <assert.h>

PHTexArgsTokenizer::PHTexArgsTokenizer(bool infoException)
	: m_options(0), m_negOptions(0), m_infile(0), m_outfile(0), m_inputTex(false), m_ignoreRC(false), m_infoException(infoException), m_option(0)
{
	m_err[0] = 0;
}

#define FREE(p) if (p) { delete p; p=0; }

int	PHTexArgsTokenizer::execOptionProc(char *arg)
{
	assert(m_option);
	OptionProc proc = m_option->proc;
	(this->*proc)(arg);
	m_option = 0;
	return 0;
}

#define BAD_OPTION(FMT, OPT) { sprintf(m_err, FMT, OPT); throw IllegalOptionException(); } 
		  
#define BAD_OPTION1(OPT) BAD_OPTION("Bad Option : -%s\n", OPT)
#define BAD_OPTION2(OPT) BAD_OPTION("Bad Option : --%s\n", OPT)

int PHTexArgsTokenizer::setSingleOption(char *arg)
{
	if (m_option) { throw IllegalOptionException(); }

	bool single = (arg[1] == 0)? true: false;
	for (char *p=arg; *p; p++) {
		const char c = *p;
		m_option = findOption(c);
		if (!m_option) { BAD_OPTION1(arg); }
		if (!single && m_option->needsNext) { BAD_OPTION1(arg); }
		if (!m_option->needsNext) { execOptionProc(0);}
	}
	return 0;
}

int PHTexArgsTokenizer::setDoubleOption(char *arg)
{
	if (m_option) { throw IllegalOptionException(); }

	m_option = findOption(arg);
	if (!m_option) { BAD_OPTION2(arg); }
	if (!m_option->needsNext) { execOptionProc(0); }
	return 0;
}

int PHTexArgsTokenizer::tokenize(int argc, char **argv)
{
	for (int i=1; i<argc; i++) {
		char *target = argv[i];
		if (*target == '-') {
			char *p = target + 1;
			if (*p == '-') { setDoubleOption(p+1); }
			else { setSingleOption(p); }
		}
		else {
			if (m_option) { execOptionProc(target); }
			else {
				inputFilename(target);
			}
		}
	}
	return 0;
}

int PHTexArgsTokenizer::tokenize(const char *optstr)
{
	const int ARYSIZE = 128;
	const int BUFSIZE = 128;

	char str[BUFSIZE];
	strcpy(str, optstr);

	char	strary[ARYSIZE][BUFSIZE];
	int	argc = 1;

	char *target = str;
	while (char *p = strtok(target, " \t\n")) {
		strcpy(strary[argc], p);
		argc++;
		target = 0;
	}

	char *argv[ARYSIZE];
	for (int i=0; i<argc; i++) {
		argv[i] = strary[i];
	}
		
	tokenize(argc, argv);

	return 0;
}

#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))

#define SINGLE_OPTION_ALL_STR "ehlntp1v"

PHTexArgsTokenizer::SingleOption *PHTexArgsTokenizer::getSingleOptions(int &n)
{
#define DECLARE(C, FUNC, NEXT, MSG) SingleOption(C, FUNC, NEXT, MSG)
	static PHTexArgsTokenizer::SingleOption options[] = {
		DECLARE('o', &PHTexArgsTokenizer::outputFilename, true, "Output file name"),
		DECLARE('d', &PHTexArgsTokenizer::debugMode, false, 0),
		DECLARE('n', &PHTexArgsTokenizer::enumerate, false, "eNumerate equations"),
		DECLARE('N', &PHTexArgsTokenizer::notEnumerate, false, "not eNumerate equations"),
		DECLARE('t', &PHTexArgsTokenizer::tag, false, "output Tag description"), 
		DECLARE('e', &PHTexArgsTokenizer::eliminateRawDescription, false, "Eliminate [[]]"),
		//		DECLARE('1', &PHTexArgsTokenizer::oneline, false, "ONE line : no return code"),
		DECLARE('l', &PHTexArgsTokenizer::sameLineNumber, false, "output same Line number as input. -1 option forced"),
		DECLARE('p', &PHTexArgsTokenizer::noEquationBlock, false, "no eQuation \\begin{...} \\end{...} description(\'p\' means the reverse of \'q\')"),
		DECLARE('i', &PHTexArgsTokenizer::ignoreRC, false, "ignore ~/.ph2texrc file description"),
		DECLARE('h', &PHTexArgsTokenizer::help, false, "print Help" ),
		DECLARE('v', &PHTexArgsTokenizer::version, false, "print Version"), 
	};
#undef DECLARE
	n = ARRAY_SIZE(options);
	return options;
}


PHTexArgsTokenizer::DoubleOption *PHTexArgsTokenizer::getDoubleOptions(int &n)
{
#define DECLARE(STR, FUNC, NEXT, MSG) DoubleOption(STR, FUNC, NEXT, MSG)
#ifdef _DEBUG
#define DECLARE_DEBUG DECLARE
#else
#define DECLARE_DEBUG(STR, FUNC, NEXT, MSG) DECLARE(STR, FUNC, NEXT, 0)	
#endif
	static PHTexArgsTokenizer::DoubleOption options[] = {
		DECLARE_DEBUG("ntt-jlatex", &PHTexArgsTokenizer::nttJlatexMode, false, "NTT-JLATEX mode [default]" ),
		DECLARE_DEBUG("ascii-platex", &PHTexArgsTokenizer::asciiPlatexMode, false, "ASCII-PLATEX mode"),
		DECLARE("itex", &PHTexArgsTokenizer::inputTex, false, "input Tex Format File"),
		DECLARE("otex", &PHTexArgsTokenizer::outputTex, false, "output Tex Format File(reserved)"),
		DECLARE("tex-desc", &PHTexArgsTokenizer::texDesc, false, "show supported TeX-like descriptions"),
		DECLARE("help", &PHTexArgsTokenizer::help, false, "print Help"),
		DECLARE("version", &PHTexArgsTokenizer::version, false, "print Version"),
	};
#undef DECLARE
	n = ARRAY_SIZE(options);
	return options;
}

PHTexArgsTokenizer::Option *PHTexArgsTokenizer::findOption(SingleOption::Key key)
{
	int	n;
	SingleOption *options = getSingleOptions(n);
	for (int i=0; i<n; i++) {
		SingleOption *option = &options[i];
		if (option->match(key)) { return option; }
	}
	return 0;
};

PHTexArgsTokenizer::Option *PHTexArgsTokenizer::findOption(DoubleOption::Key key)
{
	int    n;
	DoubleOption *options = getDoubleOptions(n);
	for (int i=0; i<n; i++) {
		DoubleOption *option = &options[i];
		if (option->match(key)) { return option; }
	}
	return 0;
}

#define PRINT_OPTION(OPTION, KEY_FMT) \
{ int n; \
  OPTION *options = get##OPTION##s(n); \
  char *fmt = "\t" KEY_FMT "\t%s\n";\
  for (int i=0; i<n; i++) { \
    OPTION &p = options[i]; char *msg = p.msg; if (msg) { printf(fmt, p.key, msg); } \
  }}

void PHTexArgsTokenizer::printHelp(const char *appname)
{
	printf("Usage : %s [-" SINGLE_OPTION_ALL_STR "] [-o] [FILE]\n", appname);
	printf("\n");
	PRINT_OPTION(SingleOption, "-%c");
	printf("\n");
	PRINT_OPTION(DoubleOption, "--%s");
#if 0
	int	n;
	{ 
		SingleOption *options = getSingleOptions(n);
		for (int i=0; i<n; i++) {
			SingleOption &p = options[i];
			char *msg = p.msg;
			if (msg) { 
				printf("\t-%c\t%s\n", p.key, msg);
			}
		}
	}

#endif
}


#ifdef PHTEX_ARGS_TEST

static void checkOptions(unsigned opts)
{
	if (opts == PHTEX_NOOPTION) { printf("no options"); return; }
	if (opts & PHTEX_ENUMERATE_EQUATION) { printf("enumerate\n"); }
	if (opts & PHTEX_OUTPUT_TAG_DESCRIPTION) { printf("output tag description\n"); }
	if (opts & PHTEX_ELIMINATE_EMBEDDED_DESCRIPTION) { printf("eliminate embedded\n"); }
	if (opts & PHTEX_ONE_LINE) { printf ("one line\n"); }
	if (opts & PHTEX_SAME_LINE_NUMBER) { printf("same line number\n"); }
	if (opts & PHTEX_NO_EQUATION_BLOCK) { printf("no equation block\n"); }
}

int main()
{
	PHTexArgsTokenizer tokenize(false);

	const char *argstr = "-e -l -N -n";
	try {
		tokenize.tokenize(argstr);
		unsigned v = tokenize.options();
		printf("Options\n");
		checkOptions(v);
		
		v = tokenize.applyNegativeOptions(v);
		printf("\nNegative Options\n");
		checkOptions(v);
	} catch(PHTexArgsTokenizer::IllegalOptionException &) {
		fprintf(stderr, tokenize.lastErr());
		return 1;
	}
	return 0;
}

#endif

