/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTexBlockConverter.cpp,v 1.12 2004/05/06 07:33:32 yoshi Exp $ */ 
#include "PHTexBlockConverter.h"
#include "matex.h"
#include "PHTexEx.h"
#include "PHTexBlock.h"
#include "PHTexArgsTokenizer.h"
#include <string.h>
#include <assert.h>

void PHTexBlockConverter::Options::set(unsigned v, unsigned nv)
{
	m_opts = v; m_negOpts = nv;
}

void PHTexBlockConverter::Options::tokenize(const char *str)
{
	const bool NOT_THROW_INFO_EXCEPTION = false;
	PHTexArgsTokenizer tokenize(NOT_THROW_INFO_EXCEPTION);
	try {
		tokenize(str);

		m_opts = tokenize.options();
		m_negOpts = tokenize.negativeOptions();

	} catch (PHTexArgsTokenizer::IllegalOptionException &) {
		fprintf(stderr, "%s\n", tokenize.lastErr());
	} catch (PHTexArgsTokenizer::Exception &) {
		assert(0);
	}
}

unsigned PHTexBlockConverter::Options::apply(unsigned v) const 
{
	v |= m_opts;
	return v & (~m_negOpts);
}

unsigned PHTexBlockConverter::BlockOptions::value() const 
{
	const unsigned v = PHTEX_NOOPTION;
	return apply(v);
}

unsigned PHTexBlockConverter::BlockOptions::apply(unsigned v) const
{
	const unsigned NO_USE_OPTIONS = PHTEX_DOCUMENT | PHTEX_ASCII_PLATEX | PHTEX_NTT_JLATEX;
	
	unsigned opts = v;
	opts = m_cmdline.apply(opts);
	opts = m_default.apply(opts);
	opts = m_thisOpts.apply(opts);
	opts &= ~NO_USE_OPTIONS;
	return opts;
}

PHTexBlockConverter::PHTexBlockConverter()
	: m_fout(0),  m_convert(false), m_firstLine(true), m_default(0), m_blockLevel(-1) {;}

bool PHTexBlockConverter::convert(FILE *in, FILE *out, unsigned opts)
{
	m_fout = out;
	m_options.commandLine(opts);
	setListenerPHTexBlock(this);
	return tokenizePHTexBlock(in);
}

void PHTexBlockConverter::defaultBlock(const char *name)
{
	if (!m_default) {
		m_default = new PHTexBlockContext::DefaultBlock;
	}
	m_default->pushBlock(name);
}

void PHTexBlockConverter::convertBlock(int n)
{
	m_context.push(new PHTexBlockContext::ConvertBlock(n));
}
void PHTexBlockConverter::ignoreBlock(int n)
{
	m_context.push(new PHTexBlockContext::IgnoreBlock(n));
}

void PHTexBlockConverter::defaultOption(const char *args)
{
	m_options.setDefaultArgs(args);
}

void PHTexBlockConverter::blockOption(const char *args)
{
	m_options.setArgs(args);
}

PHTexBlockConverter::Error PHTexBlockConverter::startBlock(const char *name, int level)
{
	if (m_convert) {
		return PHTexBlockParseListener::BPE_BLOCK_NOT_END;
	}

	if (m_default) {
		m_context.push(m_default);
		m_default = 0;
	}
	m_convert = m_context.convert(name);
	if (!m_convert) {
		fputs("\\begin{", m_fout);
		fputs(name, m_fout);
		fputs("}", m_fout);
	}
	m_firstLine = true;
	m_blockName = name;
	m_blockLevel = level;
	return (m_convert)? BPE_NOERROR: BPE_NO_CONVERSION;
}

static unsigned additioanlOptions(const char *block)
{
	unsigned opts = 0;
	if (strcmp(block, "eqnarray*") == 0) {
		opts &= ~(PHTEX_ENUMERATE_EQUATION);
	} else if( strcmp(block, "eqnarray") == 0 ||
		   strcmp(block, "equation") == 0) {
		opts |= (PHTEX_ENUMERATE_EQUATION);
	}
	return opts;
}

PHTexBlockConverter::Error PHTexBlockConverter::endBlock(const char *name, int level)
{
	if (m_convert && m_blockLevel != level) {
		return PHTexBlockParseListener::BPE_WRONG_BLOCK_DEPTH;
	}
		
	if (m_blockName.length() > 0 && m_blockName != name) {
		return BPE_BLOCK_NAME_NOT_MATCH;
	}
	Error e = BPE_NOERROR;
	
	if (m_convert) {
		unsigned opts = PHTEX_NOOPTION;
		opts |= additioanlOptions(name);
		opts = m_options.apply(opts);
		
		char *to = PHTexToTex(opts);
		if (!to) {
			return BPE_CONVERT_FAILED;
		}
		fputs(to, m_fout);
		char *err = PHTexGetLastError();
		if (err) {
			fputs(err, stderr);
			e = BPE_CONVERT_FAILED;
		}
		m_convert = false;
		m_firstLine = true;
		m_options.clearArgs();
	} else {
		fputs("\\end{", m_fout);
		fputs(name, m_fout);
		fputs("}", m_fout);
	}
	m_blockName = "";
	m_blockLevel = -1;
	return e;
}

void PHTexBlockConverter::line(int lnum, const char *str)
{
	if (m_convert) {
		if (m_firstLine) {
			PHTexSetStartLine(lnum);
			m_firstLine = false;
		}
		PHTexPushLine(str);
	}
	else { fputs(str, m_fout); }
}
