/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: PHTexExec.cpp,v 1.11 2004/01/28 14:23:41 yoshi Exp $ */ 
#include "PHTexExec.h"
#include "DrawContext.h"
#include "StdoutTextWriter.h"
#include "FileTextWriter.h"
#include "TextStdinLineReadIterator.h"
#include "Math2DTexDrawer.h"
#include "PHTexParamBase.h"
#include <stdio.h>

PHTexExec::PHTexExec(PHTexParamBase &c) :
	m_param(c), m_context(0), m_errWriter(0)
{
	;
}

DrawContext &PHTexExec::context()
{
	if (!m_context) {
		m_context = m_param.create(&m_dictionary);
	}
	return *m_context;
}

#define FREE(p) if (p) { delete p; p = 0; }

void	PHTexExec::free_()
{
	FREE(m_context);
}


void	PHTexExec::startDraw(DrawContext &context)
{
	TexType &t = context.texType();
	if (m_param.document()) {
		DrawContext::Writer &w = context.writer();
#ifdef __unix__
		if (t.isAsciiPlatex()) { w.write("\\documentclass"); }
		else { w.write("\\documentstyle"); }

		if (t.isJapanese()) { w.write("{jarticle}"); }
		else { w.write("{article}"); }
		w.write('\n');
#else
		w.write("\\documentstyle{jarticle}\n");
#endif
		w.write("\\begin{document}\n");
	}
}

void	PHTexExec::endDraw(DrawContext &context)
{
	if (m_param.document()) {
		DrawContext::Writer &w = context.writer();
		w.write("\n\\end{document}");
	}
}

Math2DTexDrawer *PHTexExec::createDrawer()
{
	Math2DTexDrawer *drawer = new Math2DTexDrawer;
	drawer->set(m_errWriter);
	return drawer;
}

bool	PHTexExec::draw()
{
	TextLineReadIterator &itr = m_param.readItr();
	Math2DTexDrawer *drawer = createDrawer();
	if (!drawer->load(itr)) {
		delete drawer;
		return false;
	}

	DrawContext &c = context();
	startDraw(c);
	drawer->draw(&c);
	int from = drawer->line();
	delete drawer;
	endDraw(c);

	if (m_param.sameLineNumber()) { 
		// ͂Əo͂̍s킹
		int	to = c.line();
		int	n = from - to;
		for (int i=0; i<n; i++) { c.writeNewline(); }
	}

	return true;
}
	
