/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Array.h,v 1.2 2001/11/06 10:29:36 yoshi Exp $ */ 
#ifndef Array_h
#define Array_h

template<class T>
class Array
{
private:
	enum { RESIZE_UNIT = 20,};
public:
	class Exception{};
	class MemoryNotAllocateException : public Exception {};
	class BufferOverFlowException : public Exception {};
private:
	int	m_size;
	int	m_arraySize;
	T*	m_array;

#if 1
	T*	alloc(int size)
	{
		T *tmp = new T [size];
		if (!tmp) { throw MemoryNotAllocateException(); }
		if (m_array) {
			for (int i=0; i<m_arraySize; i++) { tmp[i] = m_array[i]; }
			delete [] m_array;
		}
		m_array = tmp; m_arraySize = size;
		return m_array;
	}
#else
	T*	alloc(int size)
	{
		m_array = new T [size];
		if (!m_array) { throw MemoryNotAllocateException(); }
		m_arraySize = size;
		return m_array;
	}
#endif
	void resize() { alloc(m_arraySize + RESIZE_UNIT); }
public:
	Array(int size = 20) : m_arraySize(0), m_array(0) { alloc(size); m_size = 0;}
	virtual ~Array() { if (m_array) { delete [] m_array; } }

	int	size() { return m_size; }
	void	pushBack(T &t)
	{
		if (m_size >= m_arraySize) { resize(); }
		m_array[m_size++] = t;
	}
	T&	element(int i) { return m_array[i]; }
	T&	operator[](int i) { return element(i); }
};

template<class T>
class ArrayIterator
{
private:
	Array<T> *	m_array;
	int		m_curr;

	bool exist(int i) {
		return (0<=i && i< m_array->size())? true: false;
	}
public:
	ArrayIterator() : m_array(0), m_curr(-1) {;}

	void	set(Array<T> *p) { m_array = p; }

	bool	hasMoreElements()
	{
		if (m_curr == -1) { m_curr = 0; }
		else { m_curr++; }
		if (!exist(m_curr)) { m_curr = -1; return false;}
		return true;
	}

	T&	element() { return m_array->element(m_curr); }
};


#endif  // Array_h
