/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: Queue.h,v 1.1 2001/10/24 05:15:38 yoshi Exp $ */ 
#ifndef Queue_h
#define Queue_h

#include <assert.h>

template<class T>
class Queue
{
private:
	enum { RESIZE_UNIT = 20, };

private:
	T	*m_array;
	int	m_size;

	int	m_head;
	int	m_tail;

private:
	void	resize();
	bool	isFull();
	void	free_();
	int	indexIncr(int i);

public:
	Queue();
	virtual ~Queue() { free_(); }

	bool	enqueue(T &t);
	bool	dequeue();
	T &	head();

	bool	isEmpty();
	int	size();
	void	clear();
};
 
template<class T> Queue<T>::Queue()
	: m_array(0), m_size(0), m_head(0), m_tail(0)
{
	resize();
}

template<class T> void Queue<T>::free_()
{
	if (m_array) { delete [] m_array; m_array = 0; m_size = 0;}
}

template<class T> int Queue<T>::indexIncr(int i)
{
	++i %= m_size;
	return i;
}

template<class T> bool Queue<T>::enqueue(T &e)
{
	if (isFull()) { resize(); }

	m_array[m_tail] = e;
	m_tail = indexIncr(m_tail);
	return true;
}

template<class T> bool Queue<T>::dequeue()
{
	if (isEmpty()) { return false; }
	m_head = indexIncr(m_head);
	return true;
}

template<class T> T &Queue<T>::head()
{
	assert(!isEmpty());
	return m_array[m_head];
}
template<class T> int Queue<T>::size()
{
	if (m_size == 0) { return 0; }
	int n = m_tail - m_head;
	n += m_size;
	n %= m_size;
	return n;
}

template<class T> bool Queue<T>::isEmpty()
{
	return (m_head == m_tail)? true: false;
}
template<class T> bool Queue<T>::isFull()
{
	return (size() >= (m_size -1))? true: false;
}

template<class T> void Queue<T>::clear()
{
	m_head = m_tail = 0;
}

template<class T> void Queue<T>::resize()
{
	if (!isFull()) { return; }

	int newSize = m_size + RESIZE_UNIT;
	T *temp = new T [newSize];
	if (m_size != 0) {
		for (int i=0; i<m_size; i++){ temp[i] = m_array[i]; }
		delete m_array;
	}
	m_array = temp;	m_size = newSize;
}


#endif  // Queue_h

