/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: schash.c,v 1.1 2002/09/24 08:11:33 yoshi Exp $ */
/* schash .... String(key) Chaining hash */

#ifdef DEBUG_schash
#define Static /* nothing */
#else
#define Static static
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "hash.h"
#include "prime.h"
#include "schash.h"

static void *(*c_alloc)(size_t, size_t) = calloc;

void schash_set_calloc(void *(*org_calloc)(size_t, size_t))
{
  c_alloc = org_calloc;
}

kvunit *kvunit_new(void)
{
  return (kvunit *)c_alloc(1, sizeof(kvunit));
}

void kvunit_delete(kvunit *u)
{
  kvunit *now, *future;
  if (u == NULL) return;
  now = u;
  do {
    future = now->next;
		memset(now, 0, sizeof(kvunit));
    free(now);
  } while ((now = future) != NULL);
}

size_t kvunit_length(kvunit *unit)
{
  size_t len;

  len = 0;
  while (unit) {
    unit = unit->next;
    len++;
  }
  return len;
}

Static kvunit *kvunit_joint(kvunit **smallnextp, const char *k, upt v, kvunit *large, kvunit *reuse)
{
  kvunit *unit;

  /* if (smallnextp == NULL) return NULL; */
	if (reuse) unit = reuse;
  else unit = kvunit_new();
  if (unit == NULL) return NULL;

  unit->key = k;
  unit->val = v;
  *smallnextp = unit;
  unit->next = large;

  return unit;
}

Static kvunit *kvunit_insert(kvunit **headp, const char *k, upt v, kvunit *reuse)
{
  kvunit **smallnextp, *cmp;
  
  /* if (headp == NULL) return NULL; */

  smallnextp = headp;
  cmp = *headp;
  while (cmp) {
    int game = strcmp(k, cmp->key);

    if (!game) { cmp->val = v; return cmp; }
    if (game < 0) return kvunit_joint(smallnextp, k, v, cmp, reuse);
    smallnextp = &(cmp->next); cmp = cmp->next;
  }
  return kvunit_joint(smallnextp, k, v, cmp, reuse);
}

Static kvunit *kvunit_find(kvunit **headp, const char *k)
{
  kvunit *now;

  /* if (headp == NULL) return NULL; */

  now = *headp;
  while (now) {
    int game;
    game = strcmp(k, now->key);
    if (game == 0) return now;
    if (game < 0) return NULL;
    /* game > 0 */
    now = now->next;
  }
  return now;
}
  
Static kvunit *kvunit_remove(kvunit **headp, const char *k)
{
  kvunit **smallnextp, *cmp;
  
  /* if (headp == NULL) return NULL; */
  smallnextp = headp;
  cmp = *headp;
  while (cmp) {
    int game = strcmp(k, cmp->key);

    if (!game) {
      *smallnextp = cmp->next;
      cmp->next = NULL;
      return cmp;
    }

    smallnextp = &(cmp->next); cmp = cmp->next;
  }
  return cmp;
}  

schash *schash_new(size_t l)
{
  schash *self;

  if ((self = (schash *)c_alloc(1, sizeof(schash))) == NULL) return NULL;
  self->tblsiz = prime_gt(l);
  self->tbl = (kvunit **)c_alloc(self->tblsiz, sizeof(kvunit *));
  if (self->tbl == NULL) { free(self); return NULL; }

  self->hash = hash_s0;
  return self;
}

void schash_delete(schash *h)
{
  size_t i, siz;

  if (h == NULL) return;
  siz = h->tblsiz;
  for (i = 0; i < siz; i++) kvunit_delete(h->tbl[i]);
  if (h->tbl) {	memset(h->tbl, 0, siz * sizeof(kvunit *)); free(h->tbl); }
	memset(h, 0, sizeof(schash));
  free(h);
}

void schash_set_hash(schash *h, size_t (*org_hash)(const unsigned char *, size_t))
{
  if (h) h->hash = org_hash;
}

kvunit *schash_insert(schash *h, const char *k, upt v, kvunit *reuse)
{
  size_t hval;
  
  if (!h || !k) return NULL;
  hval = (h->hash)((unsigned char *)k, h->tblsiz);
  return kvunit_insert(&(h->tbl[hval]), k, v, reuse);
}

kvunit *schash_insertp(schash *h, const char *k, void *v, kvunit *reuse)
{
  upt u;
  u.p = v;
  return schash_insert(h, k, u, reuse);
}

kvunit *schash_insertt(schash *h, const char *k, type v, kvunit *reuse)
{
  upt u;
  u.t = v;
  return schash_insert(h, k, u, reuse);
}

kvunit *schash_find(schash *h, const char *k)
{
  size_t hval;
  if (!h || !k) return NULL;
  hval = (h->hash)((unsigned char *)k, h->tblsiz);
  return kvunit_find(&(h->tbl[hval]), k);
}

upt schash_find_value(schash *h, const char *k)
{
  kvunit *u = schash_find(h, k);
  return u->val;
}

kvunit *schash_remove(schash *h, const char *k)
{
  size_t hval;
  if (!h || !k) return NULL;
  hval = (h->hash)((unsigned char *)k, h->tblsiz);
  return kvunit_remove(&(h->tbl[hval]), k);
}

Static kvunit *schash_find_first(schash *h, size_t from)
{
  size_t i, siz;
  
  if (h == NULL) return NULL;

  siz = h->tblsiz;
  for (i = from; i < siz; i++) {
    kvunit *tb = h->tbl[i];
    if (tb) {
      h->tblitr = i;
      h->kvitr = tb->next;
			return tb;
    }
  }
  return NULL;
}

kvunit *schash_first(schash *h)
{
	return schash_find_first(h, 0);
}

kvunit *schash_next(schash *h)
{
  kvunit *itr;
  
  if (h == NULL) return NULL;

  itr = h->kvitr;
  if (itr) {
    h->kvitr = itr->next;
    return itr;
  }

  (h->tblitr)++;
	return schash_find_first(h, h->tblitr);
}

const char *schash_first_each(schash *h, upt *vp)
{
  upt inv;
  kvunit *u;

  if (vp == NULL) vp = &inv;

	u = schash_first(h);
	if (u) {
		*vp = u->val;
		return u->key;
	}
	
	return NULL;
}

const char *schash_next_each(schash *h, upt *vp)
{
  upt inv;
  kvunit *u;

  if (vp == NULL) vp = &inv;

	u = schash_next(h);
	if (u) {
		*vp = u->val;
		return u->key;
	}
	
	return NULL;
}

size_t schash_length(schash *h)
{
  size_t len;
  const char *k;
  
  size_t titr;
  kvunit *kitr;

  if (h == NULL) return 0;
  
  titr = h->tblitr;
  kitr = h->kvitr;

  len = 0;
  k = schash_first_each(h, NULL);
  while (k) {
    k = schash_next_each(h, NULL);
    len++;
  }
  
  h->tblitr = titr;
  h->kvitr = kitr;
  
  return len;

}

schash *schash_rehash(schash *befh, size_t newlen)
{
  schash *afth;
  kvunit *u;
  
  if ((afth = schash_new(newlen)) == NULL) return NULL;

  u = schash_first(befh);
  while (u) {
    if (!schash_insert(afth, u->key, u->val, u)) goto free;
    u = schash_next(befh);
  }

	memset(befh->tbl, 0, (befh->tblsiz) * sizeof(kvunit *));
  free(befh->tbl);
	memset(befh, 0, sizeof(schash));
  free(befh);

  return afth;

 free:
  free(afth->tbl);
  free(afth);
  return NULL;
}

int schash_histgram(schash *h, const char *format)
{
  char *informat = "%5d %3d\n";
  size_t i, siz, len;

  if (format == NULL) format = informat;
  if (h == NULL) return 0;

  siz = h->tblsiz;
  for (i = 0; i < siz; i++) {
    len = kvunit_length(h->tbl[i]);
    printf(format, i, len);
  }
  return 1;
}

#ifdef DEBUG_schash_pgm
int main(void)
{
  char *a = "foo";
  char *b = "bar";
  char *c = "quux";
  char *d = "corge";
  char *e = "";
  upt i;
  const char *k;
	kvunit *u;

  schash *h;
  int j;

  h = schash_new(4);
  i.t = 3;

  schash_insert(h, a, i, NULL);
  schash_insert(h, b, i, NULL);
  schash_insert(h, c, i, NULL);
  /* schash_insert(h, c, (upt)j); */
  schash_insertt(h, d, 0, NULL);
  schash_insertt(h, e, 0, NULL);

  k = schash_first_each(h, (upt *)&j);
  while (k) {
    printf("%s\n", k);
    k = schash_next_each(h, NULL);
  }
	printf("\n");
  u = schash_first(h);
  while (u) {
    printf("%s\n", u->key);
    u = schash_next(h);
  }

  if (schash_find(h, "aa")) abort();
  kvunit_delete(schash_remove(h, "foo"));

  k = schash_first_each(h, NULL);
  while (k) {
    printf("%s\n", k);
    k = schash_next_each(h, NULL);
  }

  schash_histgram(h, NULL);

  h = schash_rehash(h, 9);

  k = schash_first_each(h, NULL);
  while (k) {
    printf("%s\n", k);
    k = schash_next_each(h, NULL);
  }
  schash_histgram(h, NULL);
  printf("len = %d\n", schash_length(h));
  schash_delete(h);
  return 0;
}
#endif

#ifdef DEBUG_kvunit_pgm
int main(void)
{
  char *a = "foo";
  char *b = "bar";
  char *c = "quux";
  upt i;

  kvunit *head = NULL;
  kvunit *n;
  kvunit *found;

  i.t = 3;

  kvunit_insert(&head, a, i);
  kvunit_insert(&head, b, i);
  kvunit_insert(&head, c, i);
  n = head;
  while (n) {
    printf("%s %d\n", n->key, (n->val).t);
    n = n->next;
  }
  found = kvunit_find(&head, "foo");
  printf("\n%s\n", found->key);
  free(kvunit_remove(&head, "foo"));
  if (kvunit_remove(&head, "corge")) abort();
  n = head;
  while (n) {
    printf("%s %d\n", n->key, (n->val).t);
    n = n->next;
  }
  kvunit_delete(head);
  return 0;
}
#endif




