/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: AlignmentEquationDCDrawer.cpp,v 1.11 2003/10/02 03:58:59 yoshi Exp $ */ 
#include "AlignmentEquationDCDrawer.h"
#include "AlignmentEquation.h"
#include "DrawContext.h"
#include "EquationSizeManager.h"
#include "ViewBuffer.h"

static DrawContext &childContext(DrawContext &parent, DrawContext &child)
{
	child = parent;
	return child;
}

#define leftContext childContext
#define middleContext childContext
#define rightContext childContext

//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

void	AlignmentEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *whole_sz = eq->size(context);
	AlignmentEquation *align = static_cast<AlignmentEquation*>(eq);
	Equation *left = align->left();
	Equation *middle = align->middle();
	Equation *right = align->right();

	if (!context->alignSet) {
		int x = context->point.x.center(whole_sz->widthLength());
		context->align.x.setCenter(x);
		context->alignSet = true;
	}

	int	cx = context->align.x.center(whole_sz->widthLength());
	int	cy = context->point.y.center(whole_sz->heightLength());
	context->point.y.setCenter(cy);

	Equation::Size *middle_sz = 0;
	DrawContext middle_context;
	if (middle) {
		DrawContext &c = middle_context;
		middleContext(*context, c);
		c.point.x.setCenter(cx);
		middle->draw(&c);
		middle_sz = middle->size(&c);
	}

	if (left) {
		int ex = context->alignLeftEndX(whole_sz->widthLength());
		DrawContext c;
		leftContext(*context, c);
		c.point.x.setEnd(ex);
		left->draw(&c);
	}

	if (right) {
		int sx = context->alignRightStartX(whole_sz->widthLength());
		DrawContext c;
		rightContext(*context, c);
		c.point.x.setStart(sx);
		right->draw(&c);
	}
}

#define MAX(a, b) ( ((a) > (b))? (a): (b) )

#define ADD_SIZE(eq) \
{ \
	if (eq) { \
		DrawContext c; childContext(*context, c); \
		Equation::Size *sz = eq->size(context); \
		height += sz->heightLength(); width += sz->width(); \
	} \
} 


void	AlignmentEquationDCDrawer::centerWidth(Equation *eq, DrawContext *context, Length &l)
{
	AlignmentEquation *align = static_cast<AlignmentEquation*>(eq);
	Equation *middle = align->middle();

	Length	center;
	if (middle) {
		DrawContext c;
		middleContext(*context, c);
		Equation::Size *sz = middle->size(&c);
		center = sz->widthLength();
	}
	l = center;
}


void	AlignmentEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &eqSz)
{
	AlignmentEquation *align = static_cast<AlignmentEquation*>(eq);
	Equation *left = align->left();
	Equation *middle = align->middle();
	Equation *right = align->right();

	Length	height;
	int	width = 0;

	ADD_SIZE(left);

	int	cx = width;
	if (middle) {
		DrawContext c; middleContext(*context, c); 
		Equation::Size *sz = middle->size(context); 
		height += sz->heightLength();
	}
	const Length &center = context->centerWidth();
	width += center.value();
	cx += center.start_center();

	ADD_SIZE(right);

	eqSz.set(width, height.value());
	eqSz.setCenter(cx, height.start_center());
}
