/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: BracketEquationDCDrawer.cpp,v 1.13 2003/11/04 10:58:20 yoshi Exp $ */ 
#include <string.h>
#include "BracketEquationDCDrawer.h"
#include "BracketEquation.h"
#include "DrawContext.h"
#include "ViewBuffer.h"
#include "FontManager.h"
#include "EquationSizeManager.h"
#include "DCDrawType.h"
#include "EquationDCDrawerUtil.h"

static HFONT createFont(DrawContext *context, int width, int height)
{
	return context->fontMgr->getFont(FontManager::FontType::BRACKET, width, height);
}

class BracketString
{
private:
	const BracketEquation &m_bra;
public:
	BracketString(const BracketEquation &bra) : m_bra(bra) {;}
	const char *start() const
	{
		static char *strs[] = { "(", "{", "[", };
		if (!m_bra.leftVisible()) { return 0; }
		const BracketEquation::Shape s = m_bra.left();
		return (s != BracketEquation::SHAPE_NOT_SET)?
			strs[s]: 0;
	}

	const char *end() const
	{
		static char *strs[] = { ")", "}", "]", };
		if (!m_bra.rightVisible()) { return 0; }
		const BracketEquation::Shape s = m_bra.right();
		return (s != BracketEquation::SHAPE_NOT_SET)?
			strs[s]: 0;
	}

	const char *both() const
	{
		static char buf[128];
		buf[0] = 0;
		if (const char *s = start()) { strcat(buf, s); }
		if (const char *s = end()) { strcat(buf, s); }
		return buf[0]? buf: 0;
	}
};


//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

void	BracketEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *thisSize = eq->size(context);

	BracketEquation *bracket = static_cast<BracketEquation*>(eq);

	Equation *content = bracket->content();
	HDC hDC = context->buffer->getDC();

	Equation::Size *contentSize;
	if (content) {
		DrawContext content_context = *context;
		contentSize = content->size(&content_context);
		// ʂ̂ɕ`
		const int sx = context->point.x.start(thisSize->widthLength()) +
			(thisSize->width() - contentSize->width())/2;
		content_context.point.x.setStart(sx);
		const int cy = context->point.y.center(thisSize->height());
		content_context.point.y.setCenter(cy);
		content->draw(&content_context);
	}

	EquationSizeManager::Size &bracket_sz = context->sizeMgr->bracketSize(context->level);
	int bracket_height = static_cast<int>(contentSize->height()*0.7);
	HFONT hFont = ::createFont(context, bracket_sz.width, bracket_height);
	HFONT hOrigFont = (HFONT) ::SelectObject(hDC, hFont);

	int	x0, y0;

	BracketString *bstr = new BracketString(*bracket);

	if (const char *str = bstr->start()) {
		DrawContext start_context = *context;
		x0 = start_context.point.x.start(thisSize->width());
		y0 = start_context.point.y.center(thisSize->height()) - bracket_height/2;
		::TextOut(hDC, x0, y0, str, strlen(str));
	}

	if (const char *str = bstr->end()) {
		DrawContext end_context = *context;
		end_context.point.x.shift(contentSize->width() + (thisSize->width() - contentSize->width())/2);
		x0 = end_context.point.x.start(thisSize->width());
		y0 = end_context.point.y.center(thisSize->height()) - bracket_height/2;
		::TextOut(hDC, x0, y0, str, strlen(str));
	}

	::SelectObject(hDC, hOrigFont);
	::DeleteObject(hFont);

	delete bstr;
}


void	BracketEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &sz)
{
	BracketEquation *bracket = static_cast<BracketEquation*>(eq);

	Equation *content = bracket->content();
	HDC hDC = context->buffer->getDC();

	int	width = 0;
	Length height;
	Equation::Size *content_sz = content->size(context);
	width = content_sz->width(); height = content_sz->heightLength();

	BracketString *bstr = new BracketString(*bracket);
	const char *str = bstr->both();
	if (str) {

		EquationSizeManager::Size &bracket_sz = context->sizeMgr->bracketSize(context->level);
		HFONT hFont = ::createFont(context, bracket_sz.width, content_sz->height());
		HFONT hOrigFont = (HFONT) ::SelectObject(hDC, hFont);

		RectSize rs;
		EquationDCDrawerUtil::calcTextRect(hDC, str, rs);
		width += rs.width();

		// }[W͐ݒ肳ĂȂ͂Ȃ̂ŁAaddMargin ͌ĂяoȂ
		assert(marginX() == 0);
		assert(marginY() == 0);

		::SelectObject(hDC, hOrigFont);
		::DeleteObject(hFont);

	}
	delete bstr;

	sz.set(width, height.value());
	sz.setCenter(width/2, height.start_center());
}
