/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: DotEquationDCDrawer.cpp,v 1.2 2002/01/21 08:29:23 yoshi Exp $ */ 
#include "DotEquationDCDrawer.h"
#include "DotEquation.h"
#include "DrawContext.h"
#include "ViewBuffer.h"
#include "HilightSwitch.h"

class DotInterval
{
public:
	class Point {
	public:
		double x, y;
		Point() : x(0.0), y(0.0) {;}
		Point(double x_, double y_) : x(x_), y(y_) {;}
	};

private:
	Point	m_start;
	Point	m_end;
	Point	*m_middle;

public:
	DotInterval(double sx, double sy, double ex, double ey)
		: m_start(sx, sy), m_end(ex, ey), m_middle(0) {;}
	virtual ~DotInterval() { if (m_middle) { delete m_middle; m_middle = 0; } }

	Point& start() { return m_start; }
	Point& end() { return m_end; }
	Point& middle() {
		if (!m_middle) {
			m_middle = new Point( (m_start.x + m_end.x)/2, (m_start.y + m_end.y)/2 );
		}
		return *m_middle;
	}
};

#define TOP	0.5
#define BOTTOM	1.0
#define LEFT	0.0
#define RIGHT	1.0

static DotInterval dotIntervals[] =
{
	DotInterval(LEFT, 0.7, RIGHT, 0.7),	// HORIZONTAL
	DotInterval(0.5, TOP, 0.5, BOTTOM),	// VERTICAL
	DotInterval(LEFT, TOP, RIGHT, BOTTOM),	// DIAGONAL
};

#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))
#define INTERVAL_ARRAY_CHECK() assert(ARRAY_SIZE(dotIntervals) == DotEquation::DIRECTION_NUM)

#define MARGIN 1

static void DRAW_POINT(HDC hDC, int x0, int y0, int width, int height, double rx, double ry, UINT color)
{
	int x = x0 + (int)(width*rx); int y = y0 + (int)(height*ry);
	::Rectangle(hDC, x-MARGIN, y-MARGIN, x+MARGIN, y+MARGIN); 
}

void DotEquationDCDrawer::drawImage(Equation *eq, DrawContext *context,
				    int x0, int y0, int width, int height)
{
	INTERVAL_ARRAY_CHECK();
	HDC hDC = context->buffer->getDC();
	UINT color = context->hilightSwitch->color();

	DotEquation *dot = static_cast<DotEquation*>(eq);
	DotInterval &interval = dotIntervals[dot->direction()];

	DRAW_POINT(hDC, x0, y0, width, height, interval.start().x, interval.start().y, color);
	DRAW_POINT(hDC, x0, y0, width, height, interval.middle().x, interval.middle().y, color);
	DRAW_POINT(hDC, x0, y0, width, height, interval.end().x, interval.end().y, color);
}

