/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationEnumeratorEquationDCDrawer.cpp,v 1.7 2004/02/16 12:58:31 yoshi Exp $ */ 
#include "EquationEnumeratorEquationDCDrawer.h"
#include "EquationEnumeratorEquation.h"
#include "DrawContext.h"
#include "ParamEquation.h"
#include "TaggedEquation.h"
#include <stdio.h>
#include <string.h>

static DrawContext &childContext(DrawContext &parent, DrawContext &o)
{
	o = parent;
	return o;
}

static HFONT get_font(DrawContext &context)
{
	return context.getFont(FontManager::ENUMERATION);
}

const int ENUM_MARGIN_NORMAL = 200;
const int ENUM_MARGIN_MIN = 100;

//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

Equation * EquationEnumeratorEquationDCDrawer::createEnumerator(Equation *eq, int n)
{
	char buf[128];
	sprintf(buf, "(%d)", n);
	Equation *num = new ParamEquation(buf);
	if (char *tag = eq->tag()) {
		TaggedEquation *tag_eq = new TaggedEquation;
		tag_eq->tag(tag);
		tag_eq->content(num);
		num = tag_eq;
	}
	return num;
}

void EquationEnumeratorEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *whole_sz = eq->size(context);
	if (whole_sz->null()) { return; }
	bool enumerate = context->enumerate();

	int	sx = context->point.x.start(whole_sz->widthLength());
	int	cx = context->point.x.center(whole_sz->widthLength());
	int	prev_bottom = context->point.y.start(whole_sz->heightLength());

	EquationEnumeratorEquation *eq_enum = static_cast<EquationEnumeratorEquation*>(eq);

	EquationEnumeratorEquation::Container &eqs = eq_enum->equations();
	EquationEnumeratorEquation::Container::iterator i;

	for (i=eqs.begin(); i!=eqs.end(); i++) {
		Equation *e = *i;
		DrawContext c; childContext(*context, c);
		c.point.x.setCenter(cx);
		c.point.y.setStart(prev_bottom);
		e->draw(&c);
		Equation::Size *sz = e->size(&c);

		if (enumerate && e->enumerate() && !e->isDefinition()) {
			Equation *num = createEnumerator(e, context->number());
			DrawContext num_context; childContext(*context, num_context);
			num_context.point.x.setStart(sx);
			int cy = c.point.y.center(sz->heightLength());
			num_context.point.y.setCenter(cy);
			num->draw(&num_context);
			eq_enum->pushEnumerator(num);
		}

		prev_bottom = c.point.y.end(sz->heightLength());
	}

}

static void whole_size(EquationEnumeratorEquation::Container &eqs, DrawContext &context,
		       Length &width, Length &height)
{
	int	h = 0;
	EquationEnumeratorEquation::Container::iterator i;
	for (i=eqs.begin(); i!=eqs.end(); i++) {
		Equation *eq = *i;
		DrawContext c; childContext(context, c);
		Equation::Size *sz = eq->size(&c);
		width += sz->widthLength();
		h += sz->height();
	}
	height.value(h);
}

static bool enumerateNoEquation(EquationEnumeratorEquation::Container &eqs)
{
	EquationEnumeratorEquation::Container::iterator i;
	for (i=eqs.begin(); i!=eqs.end(); i++) {
		Equation *eq = *i;
		if (eq->enumerate()) { return false; }
	}
	return false;
}

#define ENUMERATOR_MARGIN_X 100

void EquationEnumeratorEquationDCDrawer::
size(Equation *eq, DrawContext *context, Equation::Size &sz)
{
	EquationEnumeratorEquation *eq_enum = static_cast<EquationEnumeratorEquation*>(eq);
	Length w, h;
	EquationEnumeratorEquation::Container &eqs = eq_enum->equations();
	if (eqs.size() == 0) { sz = 0; }
	whole_size(eqs, *context, w, h);

	int	width = w.value();
	int	height = h.value();
	if (context->enumerate() && !enumerateNoEquation(eqs)) { width += ENUMERATOR_MARGIN_X; }

	int	width_center = width - w.end_center();
	sz.set(width, height);
	sz.setCenter(width_center, height/2);
}

void EquationEnumeratorEquationDCDrawer::centerWidth(Equation *eq, DrawContext *context, Length &l)
{
	EquationEnumeratorEquation *eq_enum = static_cast<EquationEnumeratorEquation*>(eq);

	Length center;
	EquationEnumeratorEquation::Container &eqs = eq_enum->equations();
	EquationEnumeratorEquation::Container::iterator i;
	for (i=eqs.begin(); i!=eqs.end(); i++) {
		Equation *eq = *i;
		DrawContext c; childContext(*context, c);
		center += eq->centerWidth(&c);
	}
	l = center;
}

