/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationImageFactory.cpp,v 1.6 2003/09/10 07:39:56 yoshi Exp $ */ 
#include <assert.h>
#include <memory.h>
#include "EquationImageFactory.h"
#include "HilightSwitch.h"

typedef EquationImageFactory::ImageObjectCreator Creator;

void Creator::init()
{
	memset(m_images, 0x00, sizeof(m_images));
}

void Creator::free_()
{
	for (int i=0; i<Color::COLOR_TYPE_NUM; i++) {
		ImageObject **image = &m_images[i];
		if (*image) { delete *image; *image = 0;}
	}
}

ImageObject *Creator::get(EquationColor::Type t, UINT color)
{
	ImageObject **image = &m_images[t];	
	if (!*image) { *image = create(color); }
	return *image;
}


static Creator *s_creators[EquationImageFactory::IMAGE_TYPE_NUM];

void	EquationImageFactory::init()
{
	memset(s_creators, 0x00, sizeof(s_creators));
}

#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))

void	EquationImageFactory::free_()
{
	int n = ARRAY_SIZE(s_creators);
	for (int i=0; i<n; i++){
		Creator **p = &s_creators[i];
		if (*p) { delete *p; *p = 0; };
	}
}

void	EquationImageFactory::set(ImageType type, ImageObjectCreator *p)
{
	s_creators[type] = p;
}

ImageObject *EquationImageFactory::get(ImageType type, HilightSwitch *hs)
{
	Creator *creator = s_creators[type];
	assert(creator);

	const EquationColor::Type t = hs->colorType();
	const UINT color = hs->color();
	return creator->get(t, color);
}

