/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: EquationImageFactory.h,v 1.10 2004/02/19 08:36:48 yoshi Exp $ */ 
#ifndef EquationImageFactory_h
#define EquationImageFactory_h

#include "ImageObject.h"
#include "EquationColor.h"

class HilightSwitch;

class EquationImageFactory
{
public:
	enum ImageType
	{
		IMAGE_TYPE_NOT_SET = -1,
		
		IMAGE_PLUS_MINUS, IMAGE_MINUS_PLUS, IMAGE_LE, IMAGE_GE, IMAGE_NOTIN,
		IMAGE_APPROX, IMAGE_ELL,

		IMAGE_EPSILON, IMAGE_VARIANT_THETA, IMAGE_VARIANT_PI, IMAGE_VARIANT_RHO,
		IMAGE_VARIANT_SIGMA, IMAGE_VARIANT_PHI,

		IMAGE_SINGLE_LEFT_ARROW, IMAGE_SINGLE_RIGHT_ARROW, IMAGE_SINGLE_LEFT_RIGHT_ARROW,
		IMAGE_SINGLE_UP_ARROW, IMAGE_SINGLE_DOWN_ARROW,

		IMAGE_DOUBLE_LEFT_ARROW, IMAGE_DOUBLE_RIGHT_ARROW,
		IMAGE_DOUBLE_LEFT_RIGHT_ARROW, IMAGE_DOUBLE_UP_ARROW,
		IMAGE_DOUBLE_DOWN_ARROW,
 
		IMAGE_LONG_SINGLE_RIGHT_ARROW,	 IMAGE_LONG_SINGLE_LEFT_ARROW,
		IMAGE_LONG_SINGLE_LEFT_RIGHT_ARROW, IMAGE_LONG_DOUBLE_RIGHT_ARROW,
		IMAGE_LONG_DOUBLE_LEFT_ARROW, IMAGE_LONG_DOUBLE_LEFT_RIGHT_ARROW,
		
		IMAGE_MAPSTO, IMAGE_LEADSTO, IMAGE_LEFT_HARPOON_UP, IMAGE_RIGHT_HARPOON_UP,
		IMAGE_LEFT_HARPOON_DOWN, IMAGE_RIGHT_HARPOON_DOWN, IMAGE_RIGHT_LEFT_HARPOON,
		IMAGE_HOOK_LEFT, IMAGE_HOOK_RIGHT, IMAGE_SEARROW, IMAGE_SWARROW, IMAGE_NWARROW,
		IMAGE_NEARROW, IMAGE_IM, IMAGE_RE, IMAGE_IMATH, IMAGE_JMATH, IMAGE_ALEPH,
		IMAGE_HBAR, IMAGE_WP, IMAGE_EMPTYSET, IMAGE_NABLA, IMAGE_SURD,

		IMAGE_UPLUS, IMAGE_SQCAP, IMAGE_SQCUP, IMAGE_SETMINUS, IMAGE_WR,
		IMAGE_TRIANGLE_LEFT, IMAGE_TRIANGLE_RIGHT, IMAGE_LHD, IMAGE_RHD,
		IMAGE_UNLHD, IMAGE_UNRHD, IMAGE_OPLUS,	IMAGE_OMINUS, IMAGE_OTIMES,
		IMAGE_OSLASH, IMAGE_ODOT, IMAGE_AMALG, IMAGE_PREC, IMAGE_PRECEQ,
		IMAGE_SUCC, IMAGE_SUCCEQ,

		IMAGE_SUBSETEQ, IMAGE_SUPSETEQ, IMAGE_SQSUBSET, IMAGE_SQSUBSETEQ, IMAGE_SQSUPSET,
		IMAGE_SQSUPSETEQ, IMAGE_NI,IMAGE_SIMEQ,IMAGE_ASYMP,IMAGE_CONG, IMAGE_DOTEQ,
		IMAGE_MODELS, IMAGE_BOWTIE, IMAGE_JOIN, IMAGE_SMILE, IMAGE_FROWN, IMAGE_NATURAL,
		IMAGE_SPADE_SUIT, IMAGE_CLUB_SUIT, IMAGE_DIAMOND_SUIT, IMAGE_HEART_SUIT,

		IMAGE_WIDE_TILDE, IMAGE_WIDE_HAT, IMAGE_OVER_BRACE, IMAGE_UNDER_BRACE,
		IMAGE_CHECK, IMAGE_BREVE,

		IMAGE_TYPE_NUM,
	};

public:
	class ImageObjectCreator
	{
		typedef EquationColor Color;
	private:
		ImageObject	*m_images[Color::COLOR_TYPE_NUM];
		
	private:
		void	init();
		void	free_();
	protected:
		virtual ImageObject *create(UINT color) = 0;
		ImageObjectCreator() { init(); }
	public:
		virtual ~ImageObjectCreator() { free_(); }
		ImageObject *get(EquationColor::Type, UINT);
	};
public:
	static void	init();
	static void	free_();

	static void	set(ImageType type, ImageObjectCreator *p);
	static ImageObject *get(ImageType type, HilightSwitch *);
};

#endif  // EquationImageFactory_h
