/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: FontManager.cpp,v 1.9 2003/09/18 04:59:37 yoshi Exp $ */ 

#ifdef WIN32
#include <assert.h>
#include <memory.h>
#include "FontManager.h"
#include "EquationSizeManager.h"


HFONT FontManager::createFont(FontManager::FontType type, DrawLevel level, WeightType w)
{
	LOGFONT *lf = &m_logFontArray[type][level][w];
	return CreateFontIndirect(lf);
}

HFONT FontManager::createFont(FontManager::FontType type, int width, int height)
{
	LOGFONT *lf = &m_logFontArray[type][0][0];
	lf->lfWidth = width;
	lf->lfHeight = height;
	return CreateFontIndirect(lf);
}

void FontManager::init()
{
	memset(m_fontArray, 0x00, sizeof(m_fontArray));
	memset(m_fontnames, 0x00, sizeof(m_fontnames));
}


void FontManager::setFontName(FontType t, const char *fontname)
{
	if (m_fontnames[t]) { free(m_fontnames[t]);}
	m_fontnames[t] = strdup(fontname);
}

void FontManager::freeFont()
{
	for (int i=0; i<FONT_NUM; i++) {
		for (int j=0; j<DRAWLEVEL_NUM; j++){
			for (int k=0; k<WEIGHT_NUM; k++){
				HFONT *font = &m_fontArray[i][j][k];
				if (*font) { DeleteObject(*font); *font = 0;}
			}
		}
	}
}

void FontManager::free_()
{
	freeFont();

	for (int i=0; i<FONT_NUM; i++) {
		char **p = &m_fontnames[i];
		if (*p) { free(*p); *p = 0; }
	}
}

HFONT FontManager::getFont(FontType type, DrawLevel level, WeightType w)
{
	HFONT *font = &m_fontArray[type][level][w];
	if (!*font) { *font = createFont(type, level, w); }
	return *font;
}

HFONT FontManager::getFont(FontType type, int width, int height)
{
	return createFont(type, width, height);
}

static char *defaultFontnames[] =
{
	//	"Garamond",	// MATH_STRING_NORMAL
	"lr ",	// MATH_STRING_NORMAL
	"lr ",	// MATH_BIG_SYMBOL
	"lr ",	// MATH_OPERATOR
	//	"Garamond", 	// MATH_FUNCTION
	"lr ",	// MATH_FUNCTION
	//	"Bookman Old Style", // MATH_PARAM
	"lr ",	// MATH_PARAM
	"lr ", 	// BRACKET
	"lr ", 	// ARROW
	//	"Bookman Old Style", 	// ENUMERATION
	"lr ", 	// ENUM

};

class FontProperty
{
public:
	BOOL	italic;
	BYTE	charset;
};

static FontProperty fontPropArray[] =
{
	{ FALSE, SHIFTJIS_CHARSET},	// MATH_STRING_NORMAL
	{ FALSE, SHIFTJIS_CHARSET },	// MATH_BIG_SYMBOL
	{ FALSE, SHIFTJIS_CHARSET},	// MATH_OPERATOR
	{ FALSE, SHIFTJIS_CHARSET},	// MATH_FUNCTION
	{ TRUE, SHIFTJIS_CHARSET},	// MATH_PARAM
	{ FALSE, SHIFTJIS_CHARSET},	// BRACKET
	{ FALSE, SHIFTJIS_CHARSET},	// ARROW
	{ TRUE, SHIFTJIS_CHARSET},	// ENUMERATION
};

static LONG weightArray[] = { FW_NORMAL, FW_BOLD,};

#define ARRAY_SIZE(array) ( sizeof(array)/sizeof(array[0]) )

void FontManager::initFont()
{
	freeFont();
	assert(m_sizeMgr);
	assert(ARRAY_SIZE(defaultFontnames) == FONT_NUM);
	assert(ARRAY_SIZE(fontPropArray) == FONT_NUM);
	assert(ARRAY_SIZE(weightArray) == WEIGHT_NUM);
	memset(&m_logFontArray, 0x00, sizeof(m_logFontArray));
	

	for (int i=0; i<FontManager::FONT_NUM; i++){
		FontProperty	*fp = &fontPropArray[i];

		// common
		for (int j=0; j<DRAWLEVEL_NUM; j++){
			EquationSizeManager::Size sz =
				m_sizeMgr->size(static_cast<DrawLevel>(j));
			for (int k=0; k<FontManager::WEIGHT_NUM; k++){
				LONG weight = weightArray[k];

				LOGFONT *lf = &m_logFontArray[i][j][k];
				lf->lfWidth = 0;
				lf->lfHeight = sz.height;
				const char *fontname = m_fontnames[i]?
					m_fontnames[i]: defaultFontnames[i];
				strcpy(lf->lfFaceName, fontname);
				lf->lfCharSet = fp->charset;
				lf->lfItalic = (fp->italic)? TRUE: FALSE;
				lf->lfEscapement = 0;
				lf->lfOrientation = 0;
				lf->lfWeight = weight;
				lf->lfUnderline = FALSE;
				lf->lfStrikeOut = FALSE;
				lf->lfOutPrecision = OUT_DEFAULT_PRECIS;
				lf->lfClipPrecision = CLIP_DEFAULT_PRECIS;
				lf->lfQuality = DEFAULT_QUALITY;
				lf->lfPitchAndFamily = DEFAULT_PITCH;
			}
		}
	}
}

#endif // WIN32
