/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: FractionEquationDCDrawer.cpp,v 1.5 2002/09/13 08:18:05 yoshi Exp $ */ 
#include "FractionEquationDCDrawer.h"
#include "FractionEquation.h"
#include "DrawContext.h"
#include "ViewBuffer.h"
#include "EquationSizeManager.h"

static DrawContext &createContext(DrawContext &parent, DrawContext &new_context)
{
	new_context = parent;
	return new_context;
}

static int spliterMarginX(DrawContext *context)
{
	EquationSizeManager::Size &sz = context->sizeMgr->size(context->level);
	return sz.width*2;
}

//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

#define numeratorContext createContext
#define denominatorContext createContext

#define LINE(hDC, sx, sy, ex, ey) { ::MoveToEx(hDC, sx, sy, 0); ::LineTo(hDC, ex, ey); }

void	FractionEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *whole_sz = eq->size(context);
	int sx = context->point.x.start(whole_sz->widthLength());
	int cx = context->point.x.center(whole_sz->widthLength());
	int ex = context->point.x.end(whole_sz->widthLength());
	int cy = context->point.y.center(whole_sz->heightLength());

	LINE(context->buffer->getDC(), sx, cy, ex, cy);

	FractionEquation *frac_eq = static_cast<FractionEquation*>(eq);

	DrawContext num_context;
	numeratorContext(*context, num_context);
	num_context.point.x.setCenter(cx);
	num_context.point.y.setEnd(cy);
	Equation *num_eq = frac_eq->numerator();
	num_eq->draw(&num_context);

	DrawContext denomi_context;
	denominatorContext(*context, denomi_context);
	denomi_context.point.x.setCenter(cx);
	denomi_context.point.y.setStart(cy);
	Equation *denomi_eq = frac_eq->denominator();
	DEBUG_DRAW_RECT(denomi_eq, &denomi_context);
	denomi_eq->draw(&denomi_context);
}

#define MAX(a, b) ((a) > (b)? (a): (b))

void	FractionEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &eqSz)
{
	FractionEquation *frac_eq = static_cast<FractionEquation*>(eq);

	DrawContext num_context;
	numeratorContext(*context, num_context);
	Equation *num_eq = frac_eq->numerator();
	Equation::Size *num_sz = num_eq->size(&num_context);
	//	int	width = num_sz->width();
	Length	width = num_sz->widthLength();
	int	height = num_sz->height();

	DrawContext denomi_context;
	denominatorContext(*context, denomi_context);
	Equation *denomi_eq = frac_eq->denominator();
	Equation::Size *sz = denomi_eq->size(&denomi_context);

	//	width = MAX(sz->width(), width); 
	width += sz->widthLength();
	height += sz->height();
	int w = width.value() + spliterMarginX(context);

	eqSz.set(w, height);
	//	eqSz.setCenter(width/2, num_sz->height());
	eqSz.setCenter(w/2, num_sz->height());
}

