/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: HilightSwitch.cpp,v 1.10 2003/09/12 06:58:23 yoshi Exp $ */ 
#include "HilightSwitch.h"

void HilightSwitch::init()
{
	m_on = false;
	m_origPen = 0;
	m_normalPen = 0;
	m_hilightPen = 0;
	m_link = false;
	m_origColor = COLOR_NOT_SET;
}

#define FREE_OBJECT(H) if (H) { ::DeleteObject(H); H = 0; }

void HilightSwitch::free_()
{
	FREE_OBJECT(m_hilightPen);
	FREE_OBJECT(m_normalPen);
	m_origPen = 0;
}

EquationColor::Type HilightSwitch::colorType() const
{
	return (m_on)? EquationColor::COLOR_HILIGHT: EquationColor::COLOR_NORMAL;
}

UINT HilightSwitch::color() const
{
	return m_color.color(colorType());
}

EquationColor::Type HilightSwitch::normalColorType() const
{
	return (m_link)? EquationColor::COLOR_LINK: EquationColor::COLOR_NORMAL;
}

EquationColor::Type HilightSwitch::hilightColorType() const 
{
	return EquationColor::COLOR_HILIGHT;
}

UINT HilightSwitch::color(EquationColor::Type t) const 
{
	return m_color.color(t);
}

void HilightSwitch::on(HDC hDC)
{
	UINT c = color(hilightColorType());
	::SetTextColor(hDC, c);
	if (!m_hilightPen) {
		m_hilightPen = :: CreatePen(PS_SOLID, 1, c);
	} 
	m_normalPen = (HPEN) ::SelectObject(hDC, m_hilightPen);
	m_on = true;
}

void HilightSwitch::off(HDC hDC)
{
	UINT	c = color(normalColorType());
	::SetTextColor(hDC, c);
	if (!m_normalPen) {
		m_normalPen = ::CreatePen(PS_SOLID, 1, c);
	}

	::SelectObject(hDC, m_normalPen);
	m_on = false;
}

void HilightSwitch::start(HDC hDC)
{
	FREE_OBJECT(m_normalPen);
	FREE_OBJECT(m_hilightPen);
	
	UINT c = color(normalColorType());
	m_normalPen = ::CreatePen(PS_SOLID, 1, c);
	if (!m_origPen) {
		m_origPen = (HPEN)::SelectObject(hDC, m_normalPen);
	}

	if (m_origColor == COLOR_NOT_SET) {
		const UINT c = color(normalColorType());
		m_origColor = ::SetTextColor(hDC, c);
	}
	m_on = false;
}

void HilightSwitch::end(HDC hDC)
{
	if (m_origPen) {
		::SelectObject(hDC, m_origPen);
		m_origPen = 0;
	}
	if (m_origColor != COLOR_NOT_SET) {
		::SetTextColor(hDC, m_origColor);
		m_origColor = COLOR_NOT_SET;
	}
}
