/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: IntegralEquationDCDrawer.cpp,v 1.7 2002/06/10 08:00:10 yoshi Exp $ */ 
#include "IntegralEquationDCDrawer.h"
#include "IntegralEquation.h"
#include "IntegralSymbolEquation.h"
#include "DrawContext.h"
#include "EquationSizeManager.h"

Equation *IntegralEquationDCDrawer::createSymbol()
{
	return new IntegralSymbolEquation;
}

#define MAX(a, b) ( ((a)>(b))? (a): (b))

static DrawContext &symbolContext(DrawContext &parent, DrawContext &symContext)
{
	symContext = parent;
	DRAWLEVEL_DECR(symContext.level);
	return symContext;
}

static DrawContext &contentContext(DrawContext &parent, DrawContext &contentContext)
{
	contentContext = parent;
	contentContext.margin = false;
	return contentContext;
}

static DrawContext &rangeContext(DrawContext &parent, DrawContext &rangeContext)
{
	rangeContext = parent;
	DRAWLEVEL_INCR(rangeContext.level);
	rangeContext.margin = false;
	return rangeContext;
}

static int rangeXOffset(DrawContext *context)
{
	EquationSizeManager::Size &sz = context->sizeMgr->size(context->level);
	return (-sz.width);
}

void	IntegralEquationDCDrawer::
draw(Equation *eq, DrawContext *context)
{
	IntegralEquation *int_eq = static_cast<IntegralEquation*>(eq);

	Equation *symbol = createSymbol();
	DrawContext sym_context;
	symbolContext(*context, sym_context);
	symbol->draw(&sym_context);
	Equation::Size *symSize = symbol->size(&sym_context);

	DrawContext range_context;
	rangeContext(*context, range_context);
	int range_x_offset = rangeXOffset(&range_context);
	int x = sym_context.point.x.end(symSize->width()) + range_x_offset;
	range_context.point.x.setStart(x);

	Equation *lower = int_eq->lower();
	int content_x_shift = 0;
	if (lower) {
		range_context.point.y.setCenter(sym_context.point.y.end(symSize->height()));
		lower->draw(&range_context);
		Equation::Size *sz = lower->size(&range_context);
		content_x_shift = MAX(content_x_shift, sz->width() + range_x_offset);
	}
	
	Equation *upper = int_eq->upper();
	if (upper) {
		range_context.point.y.setCenter(sym_context.point.y.start(symSize->height()));
		upper->draw(&range_context);
		Equation::Size *sz = upper->size(&range_context);
		content_x_shift = MAX(content_x_shift, sz->width() + range_x_offset);
	}
		
	Equation *content = int_eq->content();
	if (content) {
		DrawContext content_context = contentContext(*context, content_context);
		content_context.point.x.shift(symSize->width() + content_x_shift);
		content_context.point.y.setCenter(context->point.y.center(symSize->height()));
		content->draw(&content_context);
	}

	delete symbol;
}


void	IntegralEquationDCDrawer::
size(Equation *eq, DrawContext *context, Equation::Size &eqSz)
{
	IntegralEquation *int_eq = static_cast<IntegralEquation*>(eq);

	Equation *symbol = createSymbol();
	DrawContext sym_context;
	symbolContext(*context, sym_context);

	Equation::Size *symSize = symbol->size(&sym_context);
	int	width = symSize->width();
	int	height = symSize->height();

	DrawContext range_context;
	rangeContext(*context, range_context);

	int	range_x_offset = rangeXOffset(&range_context);
	int	content_x_shift = 0;

	Equation *upper = int_eq->upper();
	if (upper) {
		Equation::Size *sz = upper->size(&range_context);
		content_x_shift = MAX(content_x_shift, sz->width() + range_x_offset);
		height += sz->height()/2;
	}

	Equation *lower = int_eq->lower();
	if (lower) {
		Equation::Size *sz = lower->size(&range_context);
		content_x_shift = MAX(content_x_shift, sz->width() + range_x_offset);
		height += sz->height()/2;
	}
	width += content_x_shift;

	Equation *content = int_eq->content();
	if (content) {
		DrawContext content_context;
		contentContext(*context, content_context);
		Equation::Size *sz = content->size(&content_context);
		height = MAX(sz->height(), height); width += sz->width();
	}
	delete symbol;

	eqSz.set(width, height);
}

