/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: MatrixStructuredEquationDCDrawer.cpp,v 1.7 2002/09/13 11:25:14 yoshi Exp $ */ 
#include "MatrixStructuredEquationDCDrawer.h"
#include "MatrixStructuredEquation.h"
#include "DrawContext.h"
#include "EquationSizeManager.h"

static DrawContext &elementContext(DrawContext &parent, DrawContext &elem_context)
{
	elem_context = parent;
	elem_context.margin  = true;
	return elem_context;
}

static Length *createColHeightArray(MatrixStructuredEquation *m, DrawContext *elem_context, int &n)
{
	int rowSize = m->rowSize();
	assert(rowSize != 0);
	Length *array = new Length [rowSize];

	int	colSize = m->columnSize();
	for (int row=0; row<rowSize; row++){
		Length	&height = array[row];
		for (int col=0; col<colSize; col++) {
			Equation *p = m->element(col, row); 
			if (!p) { continue; }
			Equation::Size *sz = p->size(elem_context);
			height += sz->heightLength();
		}
		assert(height.value() > 0);
	}

	n = rowSize;
	return array;
}

static Length *createRowWidthArray(MatrixStructuredEquation *m, DrawContext *elem_context, int &n)
{
	int colSize = m->columnSize();
	assert(colSize != 0);

	Length *array = new Length [colSize];

	int rowSize = m->rowSize();
	for (int col=0; col<colSize; col++){
		Length &width = array[col];
		for (int row=0; row < rowSize; row++){
			Equation *p = m->element(col, row); 
			if (!p) { continue;}
			Equation::Size *sz = p->size(elem_context);
			width += sz->widthLength();
		}
		assert(width.value() > 0);
	}
	n = colSize;
	return array;
}

#define freeArray(array) if (array) { delete [] array; }

static int elementColumnMargin(DrawContext *context)
{
	EquationSizeManager::Size &sz = context->sizeMgr->size(context->level);
	return sz.width;
}

#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

void MatrixStructuredEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	MatrixStructuredEquation *matrix = static_cast<MatrixStructuredEquation*>(eq);
	DrawContext elem_context;
	elementContext(*context, elem_context);

	Equation::Size *matSize = eq->size(context);
	int	mx0 = context->point.x.start(matSize->width());
	int	my0 = context->point.y.start(matSize->height());

	int	colHeightArraySize;
	int	rowWidthArraySize;
	Length	*colHeightArray = createColHeightArray(matrix, &elem_context, colHeightArraySize);
	Length	*rowWidthArray = createRowWidthArray(matrix, &elem_context, rowWidthArraySize);

	int	colSize = matrix->columnSize();
	int	rowSize = matrix->rowSize();
	assert(colSize == rowWidthArraySize);
	assert(rowSize == colHeightArraySize);

	int ELEMENT_COLUMN_MARGIN = elementColumnMargin(context);

	int	y0 = my0;
	for (int row=0; row<rowSize; row++){
		Length &colHeight = colHeightArray[row];
		int center_y = y0 + colHeight.start_center();
		elem_context.point.y.setCenter(center_y);
		int x0 = mx0 + ELEMENT_COLUMN_MARGIN;
		for (int col=0; col<colSize; col++){
			Length &rowWidth = rowWidthArray[col];
			Equation *p = matrix->element(col, row);
			if (p) {
				int center_x = x0 + rowWidth.start_center();
				elem_context.point.x.setCenter(center_x);
				DEBUG_DRAW_RECT(p, &elem_context);
				p->draw(&elem_context);
			}
			x0 += rowWidth.value() + ELEMENT_COLUMN_MARGIN;

		}
		y0 += colHeight.value();
	}
	freeArray(colHeightArray);
	freeArray(rowWidthArray);
}

static int sumIntArray(Length *array, int n)
{
	int sum = 0;
	for (int i=0; i<n; i++){ sum += array[i].value(); }
	return sum;
}

void MatrixStructuredEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &sz)
{
	MatrixStructuredEquation *matrix = static_cast<MatrixStructuredEquation*>(eq);

	DrawContext elem_context;
	elementContext(*context, elem_context);

	int	colHeightArraySize;
	int	rowWidthArraySize;
	Length	*colHeightArray = createColHeightArray(matrix, &elem_context, colHeightArraySize);
	Length	*rowWidthArray = createRowWidthArray(matrix, &elem_context, rowWidthArraySize);

	int	width = sumIntArray(rowWidthArray, rowWidthArraySize);
	int	height = sumIntArray(colHeightArray, colHeightArraySize);

	freeArray(colHeightArray);
	freeArray(rowWidthArray);

	int ELEMENT_COLUMN_MARGIN = elementColumnMargin(context);
	width += ELEMENT_COLUMN_MARGIN * (matrix->columnSize()+1);

	addMargin(width, height, context->margin);

	sz.set(width, height);
}
