/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: OverwriteImageEquationDCDrawer.cpp,v 1.1 2004/02/19 08:30:55 yoshi Exp $ */
#include "OverwriteImageEquationDCDrawer.h"
#include "ImageObject.h"
#include "EquationSizeManager.h"
#include "DrawContext.h"
#include "ViewBuffer.h"
#include "ContentEquation.h"
#include "EquationSizeManager.h"

static DrawContext &contentContext(DrawContext &parent, DrawContext &o)
{
	return (o = parent);
}

void OverwriteImageEquationDCDrawer::imageSize(DrawContext &c, Equation::Size &sz, int &w, int &h)
{
	EquationSizeManager::Size &normal_sz = c.sizeMgr->size(c.level);

	w = sz.width();
	const int max_w = normal_sz.width*2;
	w = (!stretch() && w > max_w)? max_w: w;
	h = normal_sz.height*0.25;
}

#define SHIFT_RIGHT_RATIO 0.25

void OverwriteImageEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	ContentEquation *parent = static_cast<ContentEquation*>(eq);
	Equation::Size *sz = parent->size(context);

	const int cx = context->point.x.center(sz->widthLength());
	const int top =  context->point.y.start(sz->heightLength());

	int imageW, imageH;

	{
		DrawContext c;
		c = *context;
		HDC hDC = c.buffer->getDC();

		const int left = context->point.x.start(sz->widthLength());
		EquationSizeManager::Size &normal_sz = c.sizeMgr->size(c.level);

		imageSize(c, *sz, imageW, imageH);
		
		const int sx = cx - imageW/2 + normal_sz.width*SHIFT_RIGHT_RATIO;
		const int sy = imageTop(context->point.y, sz->heightLength(), imageH);

		ImageObject *img = image(c);
		img->draw(hDC, sx, sy, imageW, imageH);
	}

	{
		DrawContext c;
		contentContext(*context,c);

		int sy = contentTop(context->point.y, sz->heightLength(), imageH);

		c.point.x.setCenter(cx);
		c.point.y.setStart(sy);

		Equation *content = parent->content();
		content->draw(&c);
	}
}

void OverwriteImageEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &sz)
{
	ContentEquation *parent = static_cast<ContentEquation*>(eq);
	Equation *content = parent->content();
	Equation::Size *content_sz;
	{
		DrawContext c;
		contentContext(*context, c);
		content_sz = content->size(&c);
	}
	int height = content_sz->height();
	int image_height = height * SHIFT_RIGHT_RATIO;
	int width = content_sz->width();
	
	sz.set(width, height+image_height);
	const int cy = centerY(height, image_height);
	sz.setCenter(width/2, cy);
}


int OverwriteImageTopEquationDCDrawer::centerY(int content_height, int image_height) const
{
	return image_height + content_height/2;
}

int OverwriteImageTopEquationDCDrawer::imageTop(DrawPosition &pos, Length &l, int image_height) const
{
	const int top = pos.start(l);
	return top + image_height *0.1;
}

int OverwriteImageTopEquationDCDrawer::contentTop(DrawPosition &pos, Length &l, int image_height) const
{
	const int top = pos.start(l);
	return top + image_height;
}

int OverwriteImageBottomEquationDCDrawer::imageTop(DrawPosition &pos, Length &l, int image_height) const 
{
	const int bottom = pos.end(l);
	return bottom - image_height;
}

int OverwriteImageBottomEquationDCDrawer::contentTop(DrawPosition &pos, Length &l, int image_height) const
{
	return pos.start(l);
}

int OverwriteImageBottomEquationDCDrawer::centerY(int content_height, int image_height) const
{
	return content_height/2;
}


