/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: SquareRootEquationDCDrawer.cpp,v 1.8 2003/12/26 03:59:46 yoshi Exp $ */ 
#include "SquareRootEquationDCDrawer.h"
#include "ContentEquation.h"
#include "EquationSizeManager.h"
#include "DrawContext.h"
#include "ViewBuffer.h"

static DrawContext &createContext(DrawContext &parent, DrawContext &newContext)
{
	newContext = parent;
	return newContext;
}

#define contentContext createContext
#define rootSymboContext createContext

#define LINE(hDC, sx, sy, ex, ey) { ::MoveToEx(hDC, sx, sy, 0); ::LineTo(hDC, ex, ey); }
#define LINE_TO(hDC, x, y)	::LineTo(hDC, x, y)

#define HEIGHT_RATIO 0.1
#define BAR_MARGIN 2

//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

void SquareRootEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *whole_sz = eq->size(context);
	ContentEquation *sqrt = static_cast<ContentEquation*>(eq);
	Equation *content = sqrt->content();

	DrawContext root_context;
	rootSymboContext(*context, root_context);
	EquationSizeManager::Size &sqrt_sz = root_context.sizeMgr->size(root_context.level);

	DrawContext content_context;
	contentContext(*context, content_context);
	content_context.point.x.setStart(context->point.x.start(whole_sz->width()) + (sqrt_sz.width));
	content->draw(&content_context);
	Equation::Size *content_sz = content->size(&content_context);

	HDC hDC = root_context.buffer->getDC();
	int	x, y;
	int	sx, sy, ex, ey;

	// ̃o[
	sx = context->point.x.end(whole_sz->width());
	ex = sx - content_sz->width() - sqrt_sz.width/2;
	sy = ey =
		content_context.point.y.start(content_sz->heightLength()) + (int)(sqrt_sz.height*0.1);
	LINE(hDC, sx, sy, ex, ey);

	// ɍ~
	x = ex - sqrt_sz.width/2;
	y = content_context.point.y.end(content_sz->height()) - (int)(whole_sz->height()*HEIGHT_RATIO);
	LINE_TO(hDC, x, y);

	// ɏ
	x = ex - sqrt_sz.width; 
	y = content_context.point.y.center(content_sz->height());
	LINE_TO(hDC, x, y);
}

#define MARGIN_SIZE_LEVEL(l) ((l == DRAWLEVEL0)? DRAWLEVEL1: l)

void SquareRootEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &eqSz)
{
	ContentEquation *sqrt = static_cast<ContentEquation*>(eq);
	Equation *content = sqrt->content();

	DrawContext content_context;
	contentContext(*context, content_context);
	Equation::Size *content_sz = content->size(&content_context);
	int	width = content_sz->width();
	int	height = content_sz->height();

	DrawContext root_context;
	rootSymboContext(*context, root_context);
	EquationSizeManager::Size &sz =
		root_context.sizeMgr->size(MARGIN_SIZE_LEVEL(root_context.level));
	width += sz.width*3/2;
	height += (int)(sz.height*0.1);
	eqSz.set(width, height);
	eqSz.setCenter(width/2,
		       height - content_sz->heightLength().end_center());
}

