/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: SuperSubScriptEquationDCDrawer.cpp,v 1.10 2004/02/19 12:54:11 yoshi Exp $ */ 
#include "SuperSubScriptEquationDCDrawer.h"
#include "SuperSubScriptEquation.h"
#include "FontManager.h"
#include "DrawContext.h"
#include "ViewBuffer.h"

static DrawContext &subscriptContext(DrawContext &o, DrawContext &newContext)
{
	newContext = o;
	DRAWLEVEL_INCR(newContext.level);
	newContext.margin = false;
	return newContext;
}

#define supContext subscriptContext
#define subContext subscriptContext

static DrawContext& contentContext(DrawContext &o, DrawContext &newContext)
{
	newContext = o;
	return newContext;
}

#define MAX(a, b) (((a) > (b))? (a): (b))

//#define DEBUG_DRAW
#include "EquationDCDrawerUtil.h"

void SuperSubScriptEquationDCDrawer::draw(Equation *eq, DrawContext *context)
{
	DEBUG_DRAW_RECT(eq, context);
	Equation::Size *whole_sz = eq->size(context);
	SuperSubScriptEquation *ssEq = static_cast<SuperSubScriptEquation*>(eq);
	DEBUG_DRAW_RECT(eq, context);

	Equation *sym = createSymbol();
	int	cx = context->point.x.center(whole_sz->widthLength());
	int	cy = context->point.y.center(whole_sz->heightLength());

	DrawContext sym_context;
	symbolContext(*context, sym_context);
	DEBUG_DRAW_RECT(sym, &sym_context);
	Equation::Size *symSize = sym->size(&sym_context);
	sym_context.point.x.setCenter(cx);
	sym_context.point.y.setCenter(cy);
	sym->draw(&sym_context);

	//	Equation::Size *sz;

	// ꂾƕƂɁAsymbolƏdȂĂ܂Ƃ 011127 yoshi
	// int	shiftY = symSize->height/2 - marginY();
	int	shiftY = symSize->height()/2 - marginY() + 3;

	int	symbol_center_x = sym_context.point.x.center(symSize->width());

#define DRAW_CHILD(GET_CHILD_METHOD, CONTEXT_COPY, SHIFT_Y) \
	if (Equation *child = ssEq->## GET_CHILD_METHOD ## ()) { \
		DrawContext child_context; \
		CONTEXT_COPY(*context, child_context); \
		Equation::Size *sz = child->size(&child_context); \
		child_context.point.x.setCenter(cx); \
		child_context.point.y.setCenter(cy); \
		child_context.point.y.shift(SHIFT_Y); \
		if (sz->width() <  symSize->width()) {  \
			child_context.point.x.setCenter(symbol_center_x); \
		} else { \
			child_context.point.x.setStart(context->point.x.start(whole_sz->widthLength())); \
		} \
		DEBUG_DRAW_RECT(child, &child_context); \
		child->draw(&child_context); }

	DRAW_CHILD(super, supContext, -shiftY);
	DRAW_CHILD(sub, subContext, shiftY);

	Equation *content = ssEq->content();
	if (content) {
		DrawContext content_context;
		contentContext(*context, content_context);
		Equation::Size *sz = content->size(&content_context);
		content_context.point.x.setEnd(context->point.x.end(whole_sz->widthLength()));
		content_context.point.y.setCenter(context->point.y.center(whole_sz->heightLength()));
		content->draw(&content_context);
	}

	delete sym;
}

void SuperSubScriptEquationDCDrawer::size(Equation *eq, DrawContext *context, Equation::Size &eqSz)
{
	SuperSubScriptEquation *ssEq = static_cast<SuperSubScriptEquation*>(eq);
	Equation *sym = createSymbol();
	Equation *super = ssEq->super();
	Equation *sub = ssEq->sub();
	Equation *content = ssEq->content();

	Length	width;
	int	height = 0;
	Equation::Size *sz;
	Equation **pp;
	DrawContext &(*createContextFunc)(DrawContext &, DrawContext&);

	pp = &sym;
	if (*pp) {
		DrawContext c; symbolContext(*context, c); sz = (*pp)->size(&c);
		height += sz->height(); width += sz->widthLength();
	}

	pp = &sub;
	createContextFunc = subContext;
	if (*pp) {
		DrawContext c; (*createContextFunc)(*context, c); sz = (*pp)->size(&c);
		height += sz->height(); width += sz->widthLength();
	}

	pp = &super;
	createContextFunc = supContext;
	if (*pp) {
		DrawContext c; (*createContextFunc)(*context, c); sz = (*pp)->size(&c);
		height += sz->height(); width += sz->widthLength();
	}
	int	cx = width.start_center();

	pp = &content;
	createContextFunc = contentContext;
	if (*pp) {
		DrawContext c;	(*createContextFunc)(*context, c);
		sz = (*pp)->size(&c);
		height = MAX(sz->height(), height);
		width.value(width.value() + sz->width());
	}

	int w = width.value();
	addMargin(w, height, context->margin);
	w += m_rightMargin;

	delete sym;

	eqSz.set(w, height);
	eqSz.setCenter(cx, height/2);
}
