/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: TagEquationManager.cpp,v 1.3 2001/11/14 05:58:21 yoshi Exp $ */ 
#include "TagEquationManager.h"
#include "DrawContext.h"
#include "ViewBuffer.h"
#include "Equation.h"

void	TagEquationManager::add(KeyType key, Equation *eq, DrawContext *context)
{
	EquationCmpnt *cmpnt = new EquationCmpnt;
	cmpnt->equation = eq;
	cmpnt->context = new DrawContext;
	*(cmpnt->context) = *context;
	m_map.add(key, cmpnt);
}

TagEquationManager::Iterator * TagEquationManager::find(KeyType c)
{
	Iterator *itr = new Iterator(&m_map);
	if (!itr->find(c)) {
		delete itr;
		return 0;
	}
	return itr;
}

TagEquationManager::Iterator * TagEquationManager::findAll()
{
	Iterator *itr = new Iterator(&m_map);
	if (!itr->findAll()) {
		delete itr;
		return 0;
	}
	return itr;
}

void TagEquationManager::draw(KeyType key, ViewBuffer *buffer)
{
	Iterator *itr = find(key);
	if (!itr) { return ;}
	draw(itr, buffer);
	freeIterator(itr);
}

void TagEquationManager::draw(Iterator *itr, ViewBuffer*buffer)
{
	while (itr->hasMoreElements()) {
		EquationCmpnt *cmpnt = itr->element();
		DrawContext *context = cmpnt->context;
		context->buffer = buffer;
		context->tag = false;
		cmpnt->equation->draw(context);
	}
}

void TagEquationManager::draw(ViewBuffer *buffer)
{
	Iterator *itr = findAll();
	if (!itr) { return ;}
	draw(itr, buffer);
	freeIterator(itr);
}


void TagEquationManager::free_()
{
	Iterator	itr(&m_map);
	if (!itr.findAll()) { return ; }

	while (itr.hasMoreElements()) {
		EquationCmpnt *cmpnt = itr.element();
		delete cmpnt->context;
		delete cmpnt;
	}
}

void TagEquationManager::clear()
{
	free_();
	m_map.clear();
}
