/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: ResourceBitmapObject.cpp,v 1.4 2002/08/20 09:39:13 yoshi Exp $ */ 
#include <assert.h>
#include "ResourceBitmapObject.h"

static int getColorNum(LPBITMAPINFO lpbi)
{
	switch(lpbi->bmiHeader.biBitCount) {
	case 8: return 256;
	}
	assert(0);
	return 0;
}

static LOGPALETTE *createLogPalette(LPBITMAPINFO lpbi, UINT color)
{
	int colorNum = getColorNum(lpbi);
	int size = sizeof(LOGPALETTE) + colorNum*sizeof(RGBQUAD);
	// }`Xbh̏ꍇAnew ł͂ȂHeapAllocłȂƂ
	LOGPALETTE *pLogPal = (LOGPALETTE*)::HeapAlloc(::GetProcessHeap(), HEAP_ZERO_MEMORY, size);
	pLogPal->palVersion = 0x300;
	int n = pLogPal->palNumEntries = colorNum;
	PALETTEENTRY *entries = pLogPal->palPalEntry;
	RGBQUAD *rgbs = lpbi->bmiColors;
	BYTE r = GetRValue(color);
	BYTE g = GetGValue(color);
	BYTE b = GetBValue(color);
	for (int i=0; i<colorNum; i++) {
		RGBQUAD *rgb = rgbs + i;
		PALETTEENTRY *e = entries + i;
		if (rgb->rgbRed == 0 && rgb->rgbGreen == 0 && rgb->rgbBlue == 0) {
			e->peRed = r; e->peGreen = g; e->peBlue = b;
		} else {
			e->peRed = 255; e->peGreen = 255; e->peBlue = 255; // white
		}
	}
	return pLogPal;
}

static void freeLogPalette(LOGPALETTE *p)
{
	::HeapFree(::GetProcessHeap(), 0, p);
}

static HANDLE loadBitmapResource(HINSTANCE hInst, UINT uId)
{
	return ::LoadResource(hInst,
			      FindResource(hInst, MAKEINTRESOURCE(uId), RT_BITMAP));
}

void ResourceBitmapObject::load(HINSTANCE hInst, UINT uId, UINT color)
{
	HANDLE hRes = loadBitmapResource(hInst, uId);

	LPBITMAPINFO lpbi = (LPBITMAPINFO)::LockResource(hRes);

	assert(lpbi->bmiHeader.biBitCount == 8);
	LOGPALETTE *pLogPal = createLogPalette(lpbi,color);
	m_hPalette = ::CreatePalette(pLogPal);
	freeLogPalette(pLogPal);
	FreeResource(hRes);
}

void ResourceBitmapObject::draw(HDC hDC, int x0, int y0, int w, int h)
{
	HPALETTE hOrigPalette = ::SelectPalette(hDC, m_hPalette, FALSE);
	::RealizePalette(hDC);
	HANDLE hRes = loadBitmapResource(m_hInst, m_id);
	LPBITMAPINFO lpbi = (LPBITMAPINFO)::LockResource(hRes);
	int colorNum = getColorNum(lpbi);
	unsigned char *bits =
		(unsigned char *)lpbi + lpbi->bmiHeader.biSize + colorNum*sizeof(RGBQUAD);
	int width = lpbi->bmiHeader.biWidth;
	int height = lpbi->bmiHeader.biHeight;
	StretchDIBits(hDC, x0, y0, w, h,
		      0, 0, width, height, bits, lpbi, DIB_PAL_COLORS, SRCCOPY);
	::FreeResource(hRes);
	::SelectPalette(hDC, hOrigPalette, FALSE);
	::RealizePalette(hDC);
}

#define FREE_OBJECT(h) if (h) { ::DeleteObject(h); h = 0; }

void ResourceBitmapObject::free_()
{
	FREE_OBJECT(m_hPalette);
}
