/*
 MatEx-2D Tools

 [Copyright holders' statement]
 MatEx-2D Tools is a package of softwares which are realization of
 Ishiguro's original idea of 2-dim. mathematical expression parser 
 by Saito of Mathematical Systems Inc. We distribute this software 
 from ISMLIB of the Institute of Statistical Mathematics, ISM
 ( http://www.ism.ac.jp/ismlib/index_e.html ) under the conditions 
 of Open Market License(version:OML-SW-E-1996) proposed by 
 ISM( /http://www.ism.ac.jp/ismlib/license_e.html). The OML allows 
 the free use of the source code, including redistribution and 
 modification as far as the whole of this copyright holders' statement
 is copied and shown conspicuously in the redistribution or the 
 modified version or the document attached to the products made 
 utilizing MatEx-2D Tools.

 Copyright holders take no responsibility for any damage caused by the
 use of this software.

 Copyright_OML 2004
 ISHIGURO, Makio, the Institute of Statistical Mathematics
 SAITO, Muneyoshi, Mathematical Systems Inc.
 [End of Copyright holders' statement]
*/
/* $Id: ViewBuffer.cpp,v 1.2 2001/11/14 05:55:09 yoshi Exp $ */ 
#include "ViewBuffer.h"

ViewBuffer::ViewBuffer() : m_hDC(0), m_hBitmap(0) {;}

ViewBuffer::ViewBuffer(HDC hDC, int width, int height, UINT init_color)
{
	setInitColor(init_color);
	initBI(width, height, m_bi);
	BITMAPINFO *info = getBITMAPINFO();

	m_hBitmap = CreateDIBitmap(hDC, &m_bi, 0L, 0, 0, 0);
	m_hDC = CreateCompatibleDC(hDC);
	SelectObject(m_hDC, m_hBitmap);

	clearBuffer();
	transparent();
}


ViewBuffer::ViewBuffer(ViewBuffer &o) :
	m_initColor(o.m_initColor), m_bi(o.m_bi)
{
	HDC hOrigDC = o.getDC();
	m_hBitmap = CreateDIBitmap(hOrigDC, &m_bi, 0L, 0, 0, 0);
	m_hDC = CreateCompatibleDC(hOrigDC);
	SelectObject(m_hDC, m_hBitmap);
	transparent();
	BitBlt(m_hDC, 0, 0, o.width(), o.height(), hOrigDC, 0, 0, SRCCOPY);
}
	


ViewBuffer::~ViewBuffer()
{
	if (m_hDC) { DeleteDC(m_hDC); }
	if (m_hBitmap) { DeleteObject(m_hBitmap); }
}


void ViewBuffer::setInitColor(UINT init_color)
{
	m_initColor = init_color;
}

BITMAPINFOHEADER& ViewBuffer::initBI(int width, int height,
			 BITMAPINFOHEADER &bi)
{
	memset(&bi, 0x00, sizeof(BITMAPINFOHEADER));
	bi.biSize = sizeof(BITMAPINFOHEADER);
	bi.biWidth = width;
	bi.biHeight = height;
	bi.biPlanes = 1;
	bi.biBitCount = 24;
	bi.biCompression = BI_RGB;
	return bi;
}

BITMAPINFO * ViewBuffer::getBITMAPINFO()
{
	return reinterpret_cast<BITMAPINFO*>(&m_bi);
}


void ViewBuffer::set(HDC hDC, int width, int height, UINT init_color)
{
	setInitColor(init_color);
	initBI(width, height, m_bi);
	BITMAPINFO *info = getBITMAPINFO();

	if (m_hDC) { DeleteDC(m_hDC); }
	if (m_hBitmap) { DeleteObject(m_hBitmap); }

	m_hBitmap = CreateDIBitmap(hDC, &m_bi, 0L, 0, 0, 0);
	m_hDC = CreateCompatibleDC(hDC);
	SelectObject(m_hDC, m_hBitmap);

	clearBuffer();
}
void ViewBuffer::transparent()
{
	::SetBkMode(m_hDC, TRANSPARENT);
}

void ViewBuffer::clearBuffer()
{
	HPEN hPen = CreatePen(PS_SOLID, 1, m_initColor);
	HPEN hOrigPen = (HPEN)SelectObject(m_hDC, hPen);

	LOGBRUSH	logBrush;
	logBrush.lbStyle = BS_SOLID;
	logBrush.lbColor = m_initColor;
	HBRUSH hBrush = CreateBrushIndirect(&logBrush);
	HBRUSH hOrigBrush = (HBRUSH)SelectObject(m_hDC, hBrush);

	Rectangle(m_hDC, 0, 0, width(), height());

	SelectObject(m_hDC, hOrigPen);

	SelectObject(m_hDC, hOrigBrush);

	DeleteObject(hPen);
	DeleteObject(hBrush);
}

